/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
 *
 * XRT/3d widget UIM/X integration file
 *
 *  SCCSID -- @(#)Xrt3d.cl.c	1.12    97/02/10 KL Group Inc.
 */

/*****************************************************************************
				Xrt3d.cl.c
*****************************************************************************/

#include <Xm/Xm.h>			/* Motif header file */
#include <Xm/Xrt3d.h>		/* Widget-class public header file */

#include <UxLib.h>

#ifdef	XtName
#undef	XtName
#endif

#include "veos.h"       	/* Visual Edge object system */
#include "valuesOf.h"   	/* ValuesOf & Validator functions */
#include "resource.h"
#include "resvalue.h"
#include "plist.cl.h"
#include "observer.h"
#include "cat_macros.h"

#include "Xrt3d.cl.h"		/* private XRT/3d swidget-class header file */
#include "Xrt3d_xtype.h"

/* Global variables for the Xrt3d class and its resource descriptors */

Class_t			UxC_Xrt3d = NULL_CLASS;
Class_t			UxC_Xrt3dText = NULL_CLASS;
extern Class_t	UxC_Core;

extern char *AcceptChildren();

static void Build3dText();
static void Frame3dXtObject();


static void
Xrt3d_define_resource(n, table, class)
int 				n;
Xrt3dResourceDesc	*table;
Class_t				class;
{
	binptr				prpty;
	Resource_t			*pResource;
	Xrt3dResourceDesc	*pResDesc;
	Division_t			division;
	Pass_t				pass;

	pResDesc = &(table[n]);
	prpty = UxFixed_class_prop(pResDesc->rd_name,
							   class,
							   T_PNTR,
							   pResDesc->offset);
	
	division = (Division_t) pResDesc->division;
	pass = (Pass_t) pResDesc->pass;
	
	switch(pResDesc->type->datatype.type_3d) {
	case XRT3D_UX_DOUBLE:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_UTYPE,			pResDesc->type->utype,
							 RD_XTYPE,			pResDesc->type->xtype,
							 RD_VALUESOF,		pResDesc->type->values_of,
							 RD_VALIDATOR,		pResDesc->type->validate,
							 RD_GETFUNCTION,	pResDesc->type->get_func,
							 RD_PUTFUNCTION,	pResDesc->type->put_func,
							 RD_READONLY,		pResDesc->readonly,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	case XRT3D_UX_SURFACEDATA:
	case XRT3D_UX_ZONEDATA:
	case XRT3D_UX_DISTNTABLE:
	case XRT3D_UX_CONTOURSTYLES:
	case XRT3D_UX_COLOR:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_UTYPE,			pResDesc->type->utype,
							 RD_XTYPE,			pResDesc->type->xtype,
							 RD_VALUESOF,		pResDesc->type->values_of,
							 RD_VALIDATOR,		pResDesc->type->validate,
							 RD_EDITFUNCTION,	pResDesc->type->edit_func,
							 RD_READONLY,		pResDesc->readonly,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	case XRT3D_UX_CALLBACK:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	default:
		pResource =
			UxDefineResource(RD_NAME,			pResDesc->name,
							 RD_XTNAME,			pResDesc->xt_name,
							 RD_UTYPE,			pResDesc->type->utype,
							 RD_XTYPE,			pResDesc->type->xtype,
							 RD_VALUESOF,		pResDesc->type->values_of,
							 RD_VALIDATOR,		pResDesc->type->validate,
							 RD_READONLY,		pResDesc->readonly,
							 RD_DIVISION,		division,
							 RD_PASS,			pass,
							 RD_END);
		break;
	}

	UxPutClassResource(class, prpty, pResource);
	pResDesc->prpty = prpty;
}

/* Function that defines the Xrt3d class */

void
UxRegister_Xrt3d()
{
	int		n;
	
	if (UxUndefined_class(UxC_primitive)) {
		UxRegister_primitive();
	}

	UxC_Xrt3d = UxRegister_class("Xrt3d",
								 UxC_primitive,
								 sizeof(UxXrt3d),
								 sizeof(UxXrt3dClass));

	/* General class initialization */
	UxPutUxFilename(UxC_Xrt3d, "UxXrt3d.h" );
	UxPutToolKitClass(UxC_Xrt3d, (char *) xtXrt3dWidgetClass );
	UxPutIconBitmap(UxC_Xrt3d, "xrt3d.xpm" );

	/* METHOD REGISTRATION */

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_Xrt3d);

	for (n = 0; n < num3d_res; n++) {
		if (xrt3d_res_table[n].used) {
			Xrt3d_define_resource(n, xrt3d_res_table, UxC_Xrt3d);
		}
	}

	UxInit_method(UxC_Xrt3d, UxM_UxWidgetCannotAcceptChildren, AcceptChildren);

}

/* Function that defines the Xrt3dText class */

void
UxRegister_Xrt3dText()
{
	int			n;
	Resource_t	*res;
	
	if (UxUndefined_class(UxC_RectObject)) {
		UxRegister_gadget();
	}

	UxC_Xrt3dText = UxRegister_class("Xrt3dText",
						 UxC_gadget,
						 sizeof(Xrt3dTextObject),
						 sizeof(UxXrt3dTextClass));

	/* General class initialization */
	UxPutUxFilename(UxC_Xrt3dText, "UxXrt3d.h" );
	UxPutToolKitClass(UxC_Xrt3dText, (char *) xtXrt3dTextObjectClass);
	UxPutIconBitmap(UxC_Xrt3dText, "xrt3dText.xpm" );

	/* METHOD REGISTRATION */
	
	UxInit_method(UxC_Xrt3dText, UxM_UxBuild, Build3dText);
	UxInit_method(UxC_Xrt3dText, UxM_UxDrawHandles, Frame3dXtObject);

	/* Initialization of resource descriptors */

	UxInheritResources(UxC_Xrt3dText);

	res = UxDefineResource(RD_EXAMPLE, UxGetRD_userData(UxC_Xrt3dText),
						   RD_EDITABLE, FALSE,
						   RD_END);
	UxPutClassResource(UxC_Xrt3dText, UxP_GadgetRD_userData, res);

	res = UxDefineResource(RD_EXAMPLE, UxGetRD_unitType(UxC_Xrt3dText),
						   RD_EDITABLE, FALSE,
						   RD_END);
	UxPutClassResource(UxC_Xrt3dText, UxP_GadgetRD_unitType, res);

	res = UxDefineResource(RD_EXAMPLE, UxGetRD_traversalOn(UxC_Xrt3dText),
						   RD_EDITABLE, FALSE,
						   RD_END);
	UxPutClassResource(UxC_Xrt3dText, UxP_GadgetRD_traversalOn, res);

	res = UxDefineResource(RD_EXAMPLE, UxGetRD_shadowThickness(UxC_Xrt3dText),
						   RD_EDITABLE, FALSE,
						   RD_END);
	UxPutClassResource(UxC_Xrt3dText, UxP_GadgetRD_shadowThickness, res);

	res = UxDefineResource(RD_EXAMPLE, UxGetRD_navigationType(UxC_Xrt3dText),
						   RD_EDITABLE, FALSE,
						   RD_END);
	UxPutClassResource(UxC_Xrt3dText, UxP_GadgetRD_navigationType, res);

	res = UxDefineResource(RD_EXAMPLE, UxGetRD_highlightThickness(UxC_Xrt3dText),
						   RD_EDITABLE, FALSE,
						   RD_END);
	UxPutClassResource(UxC_Xrt3dText, UxP_GadgetRD_highlightThickness, res);

	res = UxDefineResource(RD_EXAMPLE, UxGetRD_highlightOnEnter(UxC_Xrt3dText),
						   RD_EDITABLE, FALSE,
						   RD_END);
	UxPutClassResource(UxC_Xrt3dText, UxP_GadgetRD_highlightOnEnter, res);

	res = UxDefineResource(RD_EXAMPLE, UxGetRD_helpCallback(UxC_Xrt3dText),
						   RD_EDITABLE, FALSE,
						   RD_END);
	UxPutClassResource(UxC_Xrt3dText, UxP_GadgetRD_helpCallback, res);

	res = UxDefineResource(RD_EXAMPLE, UxGetRD_createManaged(UxC_Xrt3dText),
						   RD_EDITABLE, FALSE,
						   RD_END);
	UxPutClassResource(UxC_Xrt3dText, UxP_ShadowWidgetRD_createManaged, res);

	for (n = 0; n < num3dtext_res; n++) {
		if (xrt3dtext_res_table[n].used) {
			Xrt3d_define_resource(n, xrt3dtext_res_table, UxC_Xrt3dText);
		}
	}

}

char *
AcceptChildren(parent, numChildren, childClasses, children)
swidget parent;
int numChildren;
Class_t *childClasses;
swidget *children;
{
	int	 	i;
	char 	*message;
	
	message = UxWidgetCannotAcceptChildren(UxC_Core, numChildren, childClasses,
										   children);

	if (message) {
		return(message);
	}

	if (!UxIsSubclass(parent, UxC_Xrt3d)) {
		return("Parent must be an XRT/3d widget");
	}

	for (i = 0; i < numChildren; i++) {
		if (childClasses[i] != UxC_Xrt3dText) {
			return(CGETS_KL(BAD_XRT3D_CHILD));
		}
	}
	return(NULL);
}

static  void
Build3dText(newSwidget, manage)
swidget  newSwidget;
Boolean manage;
{
	/* Put the createManaged value will want. */
	UxPutInitialString(newSwidget, UxGetRD_createManaged(newSwidget),
					   "\"false\"");

	/* The Superclass method MUST be called! */
	(void) UxVoid_get_op(UxC_gadget, UxM_UxBuild) (newSwidget, manage);
}

/*-----------------------------------------------------------------------------
 * METHOD: UxDrawHandles
 *----------------------
 * NAME:        Frame3dXtObject
 * INPUT:       a 3d xtobject swidget (3dtext)
 * DESCRIPTION: No handles to draw for these types.
 * LAST REV:	Aug 97 fix17407 created.
 *---------------------------------------------------------------------------*/
static  void Frame3dXtObject(sw)
swidget sw;
{
        /* not managed un creation so no handles to draw */
}
