/*------------------------------------------------------------------------
 * $AstecId: sw-extra.c,v 1.5 2003/01/29 04:18:37 kosaka Exp $
 * $Source: /rcs/rcs2/uxext/sw-extra.c,v $
 * $Date: 1997/08/13 22:18:48 $		$Revision: 2.5.65.5 $
 *------------------------------------------------------------------------
 *		Copyright 1994, Visual Edge Software Ltd.
 *
 * ALL RIGHTS RESERVED.  This notice is intended as a precaution against
 * inadvertent publication, and shall not be deemed to constitute an 
 * acknowledgment that publication has  occurred nor to imply any waiver  
 * of confidentiality.	The year included in the notice is the year 
 * of the creation of the work.			   
 *------------------------------------------------------------------------
 * Swidget Integration Customization
 *------------------------------------------------------------------------*/

#include <veos.h>
#include <plist.h>
#include <swidget.h>
#include "UxCompound.h"
#include "utype.h"
#include "ival.h"
#include "foreach.h"
#include "cat_macros.h"

#ifdef	XRT
#include "cmpnt.cl.h"
#ifdef	HASXRT3D
#include <Xm/Xrt3d.h>
#include <Xrt3d.cl.h>
#endif	/* HASXRT3D */
#ifdef	HASXRTGEAR
#include <Xm/XrtAligner.h>
#include <Xm/XrtOutliner.h>
#include <Xm/XrtTabM.h>
#include <Xm/XrtTabB.h>
#include <Xm/XrtToolbar.h>
#include <XrtGear.cl.h>
#endif	/* HASXRTGEAR */
#ifdef	HASXRTTABLE
#include <Xm/XrtTable.h>
#include <XrtTbl.cl.h>
#endif	/* HASXRTTABLE */
#ifdef	HASXRTGAUGE
#include <Xm/XrtGauge.h>
#include <Xm/XrtGaugeAll.h>
#include <XrtGauge.cl.h>
#endif	/* HASXRTGAUGE */
#endif	/* XRT */

#ifdef PLAT

#include "cmpnt.cl.h"
#include "xk.h"

#include "xk_ds.h"
#include "uimx_cat.h"

/* X/OPEN message catalog macros. These make the code more compact. */
#define CGETS_XK(ms)    UXCATGETS(MC_XK,MS_XK_,ms)

#endif /* PLAT */


/*----------------------------------------------------------------------
 * NAME:	<UxFreezeParent>
 * DESCRIPTION:	<This function is called before chlidren recreation>
 *		It is called with the parent whose child are going to be
 *		recreated.
 * PARAMETERS: 
 *		parameter	[I|I/O|O]	parameter_description
 *		sw		I		Parent to warn.
 * NORMAL BEHAVIOR: Do Nothing
 * REVISIONS:	Jan 95 fix6802
 *--------------------------------------------------------------------*/
void
UxFreezeParent( sw)
swidget	sw;
{
#ifdef	XRT
#ifdef	HASXRTGEAR
	/*
	 * XrtSpinBox bug workarund: It looks like XrtSpinBox frees 
	 * closure without removing its associated work procedure when
	 * it is destroyed. This leads UIM/X to crash, if the memory
	 * of the freed closure is already re-used.
	 */

	/* You may want to include InitialI.h instead. */
	typedef struct _WorkProcRec {
		XtWorkProc proc;
		XtPointer closure;
		struct _WorkProcRec *next;
		XtAppContext app;
	} WorkProcRec;

	typedef struct _XtAppStruct {
		void *next;
		void *process;
		void *destroy_callbacks;
		void **list;
		void *timerQueue;
		WorkProcRec *workQueue;
	} *XtAppStructPtr;

	/* You might think we should use the application
	   context for the widget correspinding to swidget sw.
	   However, it does not work since sw is NULL when
	   the work procedure to be deleted exists. */

	XtAppStructPtr app = (XtAppStructPtr)UxAppContext;
	WorkProcRec *w = app->workQueue, *next;
	int delete;

	while(w){
		delete = (*(w->proc)) (w->closure);
		if(delete){
			next = w->next;
			XtRemoveWorkProc((XtWorkProcId)w);
			w = next;
		}
		else {
			w = w->next;
		}
	}
#endif	/* HASXRTGEAR */
#endif	/* XRT */
}

/*----------------------------------------------------------------------
 * NAME:	<UxUnfreezeParent>
 * DESCRIPTION:	<This function is called after children recreation>
 *		It is called with the parent.
 * PARAMETERS: 
 *		parameter	[I|I/O|O]	parameter_description
 *		sw		I		Parent to warn.
 * NORMAL BEHAVIOR: Do Nothing
 * REVISIONS:	Jan 95 fix6802
 *--------------------------------------------------------------------*/
void
UxUnfreezeParent( sw)
swidget	sw;
{
}

/*----------------------------------------------------------------------
 * NAME:	<UxOnlySetValuesNeeded>
 * DESCRIPTION:	<This function is called whenever UIM/X has to destroy 
 *		and recreate a swidget.> If the swidget can be updated
 *		only by doing a SetValues, then so be it. The callee
 *		has to set all the resources required on the swidget.
 *		
 * PARAMETERS: 
 *		parameter	[I|I/O|O]	parameter_description
 *		sw		I		swidget to update
 * RETURN:	True	- Values updated, no need to recreate.
 *		False	- Recreate the widget.
 * NORMAL BEHAVIOR: Returns False
 * REVISIONS:	Jan 95 fix6802
 *--------------------------------------------------------------------*/
int	
UxOnlySetValuesNeeded(sw)
swidget	sw;
{
	return False;
}

/*----------------------------------------------------------------------
 * NAME:	<UxIsCustomConstraint>
 * DESCRIPTION:	<Called when UIM/X needs to know if recreating a child
 *		should cause any following children in the children list to be 
 *		recreated.>
 *		This is a case where a RowColumn-type widget would need
 *		to have all the following children to be recreated to
 *		preserve a certain order.
 * PARAMETERS: 
 *		parameter	[I|I/O|O]	parameter_description
 *		sw		I		swidget to ask for
 *						importance of order.
 * RETURN:	True	- UIM/X must recreate following children
 *		False	- Only recreate that one child.
 * NORMAL BEHAVIOR: returns False
 * LAST REV:	April97 fix16794 XRT constraints.
 *--------------------------------------------------------------------*/
int
UxIsCustomConstraint( sw)
swidget	sw;
{
#ifdef	XRT
#ifdef	HASXRTGEAR
        Widget w;

        w = UxGetWidget(sw);

        if( XmIsXrtToolbar(w) || XmIsXrtTabManager(w) || XmIsXrtAligner(w)) {
                return TRUE;
        }
#endif	/* HASXRTGEAR */
#endif	/* XRT */
	return False;
}

/*----------------------------------------------------------------------
 * NAME:	<UxCustomCreateFunction>
 * DESCRIPTION:	<Function that can change the way a widget is created
 *		by calling an alternate create function.> Should not modify
 *		the hierarchy, but can insert extra children in between,
 *		etc.
 * PARAMETERS: 
 *		parameter	[I|I/O|O]	parameter_description
 *		sw		I		swidget whose widget is
 *						is created
 *		xparent		I		The proposed X Parent
 *		arglist		I		The pre-built arglist
 *		num_args	I		How many args
 * RETURN:	Widget	- This is the widget to put into the swidget.
 *		NULL	- Proceed with normal processing.
 * NORMAL BEHAVIOR: Returns NULL
 * REVISIONS:	Jan 95 fix6802
 *--------------------------------------------------------------------*/
Widget
UxCustomCreateFunction( sw, xparent, arglist, num_args)
swidget		sw;
Widget		xparent;
Arg		*arglist;
int		num_args;
{
	return NULL;
}

/*----------------------------------------------------------------------
 * NAME:	<UxCustomEventMask>
 * DESCRIPTION:	<This function is called as part of UxUpdateDisplay>
 *
 * PARAMETERS:
 *		parameter	[I|I/O|O]	parameter_description
 * NORMAL BEHAVIOR: Return 0 Mask
 * REVISIONS:	Dec 95	fix11296
 *--------------------------------------------------------------------*/
long
UxCustomEventMask(Void)
{
	return 0L;
}

/*----------------------------------------------------------------------
 * NAME:        <UxChildCannotAcceptParent>
 * DESCRIPTION: Can be used by builder engine products to refuse
 *		certain parents for certain children (e.g. in CPT
 *		meuu panes and menu items can only be added to
 *		specific parents).
 * PARAMETERS:  swidget	parent, 
 *		Class_t child_class,
 *		swidget child
 *
 * RETURN:      True if child doesn't like the parent.
 * LAST REV:    Jun 97	fix17157 - included checking for Xrt widgets.
 *--------------------------------------------------------------------*/
char*
UxChildCannotAcceptParent( parent, child_class, child )
	swidget parent;
	Class_t child_class;
	swidget child;
{
	char *msg = NULL;
	Widget parent_wgt;

#ifdef PLAT

	/*----------------------------------------------------
	 * First check for an instance. If the child is an
	 * instance then it may be a CPT component. We are
	 * interested in MenuPanes and MenuItems.
	 *---------------------------------------------------*/

        if (child && UxIsSubclass(child, UxC_instance))
        {
                char *cname = UxGetComponentName((instance*)child);

                if (cname && (strncmp(cname,"MenuPane",8) == 0))
		{
			/*--------------------------------------------
			 * We got a MenuPane as a child. It can only
			 * be placed on a primary or a secondary
			 * (i.e. Form subclass) or another MenuPane.
			 *-------------------------------------------*/

			if (UxIsSubclass(parent, UxC_instance))
			{
				char *parentname =
					 UxGetComponentName((instance*)parent);

				if (parentname && ((XkIsFormSubclass(parent) ||
				    strncmp(parentname,"MenuPane",8) == 0)))
				{
					return (char*)NULL;
				}
				else
				{
					return CGETS_XK(BAD_MENUPANE_PARENT);
				}
			}
			else
			{
				return CGETS_XK(BAD_MENUPANE_PARENT);
			}
		}
		else if (cname && (strncmp(cname,"MenuItem",8) == 0))
		{
			/*--------------------------------------------
			 * We got a MenuItem as a child. It can only
			 * be placed on a MenuPane.
			 *-------------------------------------------*/

			if (UxIsSubclass(parent, UxC_instance))
			{
				char *parentname =
					 UxGetComponentName((instance*)parent);

				if ( parentname &&
				     (strncmp(parentname,"MenuPane",8) == 0))
                		{
					return (char*)NULL;
				}
				else
				{
					return CGETS_XK(BAD_MENUITEM_PARENT);
				}
			}
			else
			{
				return CGETS_XK(BAD_MENUITEM_PARENT);
			}
		}
        }
#endif /* PLAT */

#ifdef	XRT
	extern swidget UxChildSite();

	/*-----------------------------------------------------
	 * There are quite a few Xrt Widgets and Objects that
	 * are specifically designed to work with other Xrt
	 * Widgets. They should not be parented on anything but
	 * the parents they were designed for.
	 *----------------------------------------------------*/

	/*-----------------------------------------------------
	 * First, we have to determine the real parent widget
	 *----------------------------------------------------*/

	if (parent)
	{
		if (UxIsSubclass(parent, UxC_instance))
		{
			parent = UxGetLittleBrother((instance *)parent);
			parent = UxChildSite(parent);
		}
		parent_wgt = UxGetWidget(parent);
	}
	else
	{
		parent_wgt = (Widget) NULL;
	}

#ifdef HASXRT3D
	if (UxClassIsSubclass(child_class, UxC_Xrt3dText))
	{
		/*-----------------------------------------------
		 * Xrt3dTextObject can only be a child of Xrt3d,
		 * and it cannot be a top level.
		 *----------------------------------------------*/

		if (!(parent_wgt && XtIsXrt3d(parent_wgt)))
		{
			msg =  CGETS_KL(BAD_XRT3DTEXT_PARENT);
		}
	}
#endif	/* HASXRT3D */
#ifdef	HASXRTTABLE
#if	defined(HASXRT3D)
	else
#endif
        if (UxClassIsSubclass(child_class, UxC_XrtLabel))
	{
		/*-----------------------------------------------
		 * XrtLabel can only be a child of XrtTable, and
		 * it cannot be a top level.
		 *----------------------------------------------*/

		if (!(parent_wgt && XtIsXrtTable(parent_wgt)))
		{
			msg = CGETS_KL(BAD_XRTLABEL_PARENT);
		}
	}
#endif /* HASXRTTABLE */
#ifdef HASXRTGEAR
#if	defined(HASXRT3D) || defined(HASXRTTABLE)
	else
#endif
	if (UxClassIsSubclass(child_class, UxC_XrtColumn) ||
	    UxClassIsSubclass(child_class, UxC_XrtNodeStyle))
	{
		/*-----------------------------------------------
		 * XrtColumn and XrtNodeStye can only be 
		 * a child of XrtOutliner. They cannot be top
		 * levels either.
		 *----------------------------------------------*/

		if (!(parent_wgt && XmIsXrtOutliner(parent_wgt)))
		{
			if (UxClassIsSubclass(child_class, UxC_XrtColumn))
			{
				msg = CGETS_KL(BAD_XRTCOLUMN_PARENT);
			}
			else
			{
				msg = CGETS_KL(BAD_XRTNODESTYLE_PARENT);
			}
		}
	}
	else if (UxClassIsSubclass(child_class, UxC_XrtTabB))
	{
		/*-----------------------------------------------
		 * XrtTabB can only be the child of an XrtTabM,
		 * and it cannot be a top level.
		 *---------------------------------------------*/

		if (!(parent_wgt && XmIsXrtTabManager(parent_wgt)))
		{
			msg = CGETS_KL(BAD_XRTTABB_PARENT);
		}
	}
#endif	/* HASXRTGEAR */
#ifdef HASXRTGAUGE
#if	defined(HASXRT3D) || defined(HASXRTTABLE) || defined(HASXRTGEAR)
	else
#endif
	if (UxClassIsSubclass(child_class, UxC_XrtGaCenter)) {

          if( !(parent_wgt && XmIsXrtGaCircularGauge(parent_wgt)) )
	  {
		msg = CGETS_KL(BAD_XRTGACENTER_PARENT);
	  }

	}
	else if (UxClassIsSubclass(child_class, UxC_XrtGaCircularScale) ) {

          if( !(parent_wgt &&
		(XmIsXrtGaCircularDial(parent_wgt)
		|| XmIsXrtGaCircularGauge(parent_wgt)
		|| XmIsXrtGaPercent(parent_wgt))) )
	  {
		msg = CGETS_KL(BAD_XRTGACIRCULARSCALE_PARENT);
	  }

	}
	else if (UxClassIsSubclass(child_class, UxC_XrtGaCounter)) {

          if( !(parent_wgt &&
		(XmIsXrtGaCircularDial(parent_wgt)
		|| XmIsXrtGaCircularGauge(parent_wgt))) )
	  {
		msg = CGETS_KL(BAD_XRTGACOUNTER_PARENT);
	  }

	}
	else if (UxClassIsSubclass(child_class, UxC_XrtGaFrame)) {

          if( !(parent_wgt &&
		( XmIsXrtGaCircularDial(parent_wgt)
		|| XmIsXrtGaCircularGauge(parent_wgt)
		|| XmIsXrtGaLinearDial(parent_wgt)
		|| XmIsXrtGaLinearGauge(parent_wgt)
		|| XmIsXrtGaPercent(parent_wgt))) )
	  {
		msg = CGETS_KL(BAD_XRTGAFRAME_PARENT);
	  }

	}
	else if (UxClassIsSubclass(child_class, UxC_XrtGaIcon)) {

          if( !(parent_wgt && XmIsXrtGaLEDIcon(parent_wgt)) )
	  {
		msg = CGETS_KL(BAD_XRTGAICON_PARENT);
	  }

	}
	else if (UxClassIsSubclass(child_class, UxC_XrtGaIndicator)) {

          if( !(parent_wgt &&
		( XmIsXrtGaLinearDial(parent_wgt)
		|| XmIsXrtGaLinearGauge(parent_wgt))) )
	  {
		msg = CGETS_KL(BAD_XRTGAINDICATOR_PARENT);
	  }

	}
	else if (UxClassIsSubclass(child_class, UxC_XrtGaItem)) {

          if( !(parent_wgt && XmIsXrtGaPercent(parent_wgt)) )
	  {
		msg = CGETS_KL(BAD_XRTGAITEM_PARENT);
	  }

	}
	else if (UxClassIsSubclass(child_class, UxC_XrtGaLinearScale)) {

          if( !(parent_wgt &&
		( XmIsXrtGaLinearDial(parent_wgt)
		|| XmIsXrtGaLinearGauge(parent_wgt)
		|| XmIsXrtGaPercent(parent_wgt))) )
	  {
		msg = CGETS_KL(BAD_XRTGALINEARSCALE_PARENT);
	   }

	}
	else if (UxClassIsSubclass(child_class, UxC_XrtGaMarker)) {

          if( !(parent_wgt && XmIsXrtGaCircularDial(parent_wgt)) )
	  {
		msg = CGETS_KL(BAD_XRTGAMARKER_PARENT);
	  }

	}
	else if (UxClassIsSubclass(child_class, UxC_XrtGaNeedle)) {

          if( !(parent_wgt && XmIsXrtGaCircularGauge(parent_wgt)) )
	  {
		msg = CGETS_KL(BAD_XRTGANEEDLE_PARENT);
	  }

	}
	else if (UxClassIsSubclass(child_class, UxC_XrtGaRange)) {

          if( !(parent_wgt &&
		( XmIsXrtGaCircularDial(parent_wgt)
		|| XmIsXrtGaCircularGauge(parent_wgt)
		|| XmIsXrtGaLinearGauge(parent_wgt))) )
	  {
		msg = CGETS_KL(BAD_XRTGARANGE_PARENT);
	  }

	}
	else if (UxClassIsSubclass(child_class, UxC_XrtGaState)) {

          if( !(parent_wgt && XmIsXrtGaCircularDial(parent_wgt)) )
	  {
		msg = CGETS_KL(BAD_XRTGASTATE_PARENT);
	  }

	}
	else if (UxClassIsSubclass(child_class, UxC_XrtGaSwitch)) {

          if( !(parent_wgt && XmIsXrtGaCircularDial(parent_wgt)) )
	  {
		msg = CGETS_KL(BAD_XRTGASWITCH_PARENT);
	  }

	}
	else if (UxClassIsSubclass(child_class, UxC_XrtGaText)) {

          if( !(parent_wgt &&
		( XmIsXrtGaCircularDial(parent_wgt)
		|| XmIsXrtGaCircularGauge(parent_wgt)
		|| XmIsXrtGaLinearDial(parent_wgt)
		|| XmIsXrtGaLinearGauge(parent_wgt)
		|| XmIsXrtGaPercent(parent_wgt))) )
	  {
		msg = CGETS_KL(BAD_XRTGATEXT_PARENT);
	  }

	}
	else if (UxClassIsSubclass(child_class, UxC_XrtGaTick)) {

          if( !(parent_wgt &&
		( XmIsXrtGaCircularDial(parent_wgt)
		|| XmIsXrtGaCircularGauge(parent_wgt)
		|| XmIsXrtGaLinearDial(parent_wgt)
		|| XmIsXrtGaLinearGauge(parent_wgt))) )
	  {
		msg = CGETS_KL(BAD_XRTGATICK_PARENT);
	  }

	}
#endif	/* HASXRTGAUGE */
#endif	/* XRT */

        return msg;
}

/*----------------------------------------------------------------------
 * NAME:        UxIsBuilderEngineImposed
 * DESCRIPTION: Can be used by builder engine products to force
 *		recreation of certain wdiegt classes whenever a child
 *		is being recreated.
 * PARAMETERS:  swidget	parent, 
 * RETURN:      1 if parent should be recreated.
 * LAST REV:    Aug 97	fix17396	Created
 *--------------------------------------------------------------------*/
int
UxIsBuilderEngineImposed( parent )
	swidget parent;
{
#ifdef	XRT
#ifdef	HASXRTGEAR
	if (UxIsSubclass( parent, UxC_XrtTabM ))
	{
		return 1;
	}
#endif	/* HASXRTGEAR */
#ifdef	HASXRTGAUGE

#if FALSE
	/* When you update the XRT/gauge objects, you need to recreate
	   their parent widgets. This code fragment can do this
	   automatically but has side effect; when you add new objects
	   to XRT/gauge widget, the interface may be hidden (You will
	   have to click the interface icon on the Project Window to
	   show it) */

	if (UxIsSubclass( parent, UxC_XrtGaCircularDial )
	    || UxIsSubclass( parent, UxC_XrtGaCircularGauge )
	    || UxIsSubclass( parent, UxC_XrtGaLEDIcon )
	    || UxIsSubclass( parent, UxC_XrtGaLinearDial )
	    || UxIsSubclass( parent, UxC_XrtGaLinearGauge )
	    || UxIsSubclass( parent, UxC_XrtGaPercent ))
	{
		return 1;
	}
#endif

#endif	/* HASXRTGEAR */
#endif	/* XRT */

	/* by default nothing is imposed */
	return 0;
}
