/* $AstecId: XrtGauge_xtype.c,v 1.2 2003/01/29 04:20:46 kosaka Exp $ */
/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
 *
 * XRT/gauge widget UIM/X integration file
 *
 *  SCCSID -- @(#)XrtGauge_xtype.c	1.2    99/12/08 KL Group Inc.
 */

/******************************************************************************
				XrtGauge_xtype.c
******************************************************************************/

#include <UxLib.h>
#include <vtypes.h>
#include <utype.h>
#include <ctype.h>
#include <string.h>
#include <time.h>

#ifdef DESIGN_TIME

#include "veos.h"       		/* Visual Edge object system */
#include "valuesOf.h"   		/* ValuesOf & Validator functions */
#include "resource.h"
#include "resvalue.h"
#include "plist.cl.h"
#include "observer.h"

#endif /* DESIGN_TIME */

#ifdef HASXRTGAUGE

#include <Xm/XrtGauge.h>

/*temporary include all .h files*/
#include <Xm/XrtGaugeAll.h>

#include "Xrt_restable.h"

#include <cat_macros.h>

/* Begin new XRT/gauge types */

int	    UxXT_XrtGaAlignment;
int	    UxXT_XrtGaBorder;
int	    UxXT_XrtGaCalcMode;
int	    UxXT_XrtGaDirection;
int	    UxXT_XrtGaDisplayMode;
int	    UxXT_XrtGaDouble;
int	    UxXT_XrtGaImage;
int	    UxXT_XrtGaImageLayout;
int	    UxXT_XrtGaIndicatorType;
int	    UxXT_XrtGaInteractionType;
int	    UxXT_XrtGaLinePattern;
int	    UxXT_XrtGaMarkerType;
int	    UxXT_XrtGaNeedleType;
int	    UxXT_XrtGaObject;
int	    UxXT_XrtGaObjectList;
int	    UxXT_XrtGaOffsetType;
int	    UxXT_XrtGaShape;
int	    UxXT_XrtGaSizingPolicy;
int	    UxXT_XrtGaStringMode;
int	    UxXT_XrtGaSwitchType;
int	    UxXT_XrtGaTickType;
int	    UxXT_XrtGaValueType;


/* Define three static arrays for each enumerated type */

/* XrtGaAlignment */
static char *uXrtGaAlignment[] = {
	"AlignmentBeginning",
	"AlignmentCenter",
	"AlignmentEnd"
};

static unsigned char xXrtGaAlignment[] = {
	XRTGA_ALIGNMENT_BEGINNING,
	XRTGA_ALIGNMENT_CENTER,
	XRTGA_ALIGNMENT_END
};

static char *dXrtGaAlignment[] = {
	"XRTGA_ALIGNMENT_BEGINNING",
	"XRTGA_ALIGNMENT_CENTER",
	"XRTGA_ALIGNMENT_END"
};

/* XrtGaBorder */
static char *uXrtGaBorder[] = {
	"BorderNone",
	"BorderPlain",
	"Border3dIn",
	"Border3dOut",
	"BorderEtchedIn",
	"BorderEtchedOut",
	"BorderFrameIn",
	"BorderFrameOut",
	"BorderBevel",
	"BorderShadow"
};

static unsigned char xXrtGaBorder[] = {
	XRTGA_BORDER_NONE,
	XRTGA_BORDER_PLAIN,
	XRTGA_BORDER_3D_IN,
	XRTGA_BORDER_3D_OUT,
	XRTGA_BORDER_ETCHED_IN,
	XRTGA_BORDER_ETCHED_OUT,
	XRTGA_BORDER_FRAME_IN,
	XRTGA_BORDER_FRAME_OUT,
	XRTGA_BORDER_BEVEL,
	XRTGA_BORDER_SHADOW
};

static char *dXrtGaBorder[] = {
	"XRTGA_BORDER_NONE",
	"XRTGA_BORDER_PLAIN",
	"XRTGA_BORDER_3D_IN",
	"XRTGA_BORDER_3D_OUT",
	"XRTGA_BORDER_ETCHED_IN",
	"XRTGA_BORDER_ETCHED_OUT",
	"XRTGA_BORDER_FRAME_IN",
	"XRTGA_BORDER_FRAME_OUT",
	"XRTGA_BORDER_BEVEL",
	"XRTGA_BORDER_SHADOW"
};

/* XrtGaCalcMode */
static char *uXrtGaCalcMode[] = {
	"CalcModePercent",
	"CalcModeRatio"
};

static unsigned char xXrtGaCalcMode[] = {
	XRTGA_CALC_MODE_PERCENT,
	XRTGA_CALC_MODE_RATIO
};

static char *dXrtGaCalcMode[] = {
	"XRTGA_CALC_MODE_PERCENT",
	"XRTGA_CALC_MODE_RATIO"
};


/* XrtGaDirection */
static char *uXrtGaDirection[] = {
	"DirectionForward",
	"DirectionBackward"
};

static unsigned char xXrtGaDirection[] = {
	XRTGA_DIRECTION_FORWARD,
	XRTGA_DIRECTION_BACKWARD
};

static char *dXrtGaDirection[] = {
	"XRTGA_DIRECTION_FORWARD",
	"XRTGA_DIRECTION_BACKWARD"
};

/* XrtGaDisplayMode */
static char *uXrtGaDisplayMode[] = {
	"DisplayModeMagnitude",
	"DisplayModeBinary"
};

static unsigned char xXrtGaDisplayMode[] = {
	XRTGA_DISPLAY_MODE_MAGNITUDE,
	XRTGA_DISPLAY_MODE_BINARY
};

static char *dXrtGaDisplayMode[] = {
	"XRTGA_DISPLAY_MODE_MAGNITUDE",
	"XRTGA_DISPLAY_MODE_BINARY"
};

/* XrtGaImageLayout */
static char *uXrtGaImageLayout[] = {
	"ImageLayoutCentered",
	"ImageLayoutTiled"
};

static unsigned char xXrtGaImageLayout[] = {
	XRTGA_IMAGE_LAYOUT_CENTERED,
	XRTGA_IMAGE_LAYOUT_TILED
};

static char *dXrtGaImageLayout[] = {
	"XRTGA_IMAGE_LAYOUT_CENTERED",
	"XRTGA_IMAGE_LAYOUT_TILED"
};

/* XrtGaIndicatorType */
static char *uXrtGaIndicatorType[] = {
	"IndicatorTypeRectangle",
	"IndicatorTypeCircle",
	"IndicatorTypeTriangle",
	"IndicatorTypePointer",
	"IndicatorTypeImage"
};

static unsigned char xXrtGaIndicatorType[] = {
	XRTGA_INDICATOR_TYPE_RECTANGLE,
	XRTGA_INDICATOR_TYPE_CIRCLE,
	XRTGA_INDICATOR_TYPE_TRIANGLE,
	XRTGA_INDICATOR_TYPE_POINTER,
	XRTGA_INDICATOR_TYPE_IMAGE
};

static char *dXrtGaIndicatorType[] = {
	"XRTGA_INDICATOR_TYPE_RECTANGLE",
	"XRTGA_INDICATOR_TYPE_CIRCLE",
	"XRTGA_INDICATOR_TYPE_TRIANGLE",
	"XRTGA_INDICATOR_TYPE_POINTER",
	"XRTGA_INDICATOR_TYPE_IMAGE"
};

/* XrtGaInteractionType */
static char *uXrtGaInteractionType[] = {
	"InteractionTypeNone",
	"InteractionTypeRelativeToMouse",
	"InteractionTypeSnapToMouse"
};

static unsigned char xXrtGaInteractionType[] = {
	XRTGA_INTERACTION_TYPE_NONE,
	XRTGA_INTERACTION_TYPE_RELATIVE_TO_MOUSE,
	XRTGA_INTERACTION_TYPE_SNAP_TO_MOUSE
};

static char *dXrtGaInteractionType[] = {
	"XRTGA_INTERACTION_TYPE_NONE",
	"XRTGA_INTERACTION_TYPE_RELATIVE_TO_MOUSE",
	"XRTGA_INTERACTION_TYPE_SNAP_TO_MOUSE"
};

/* XrtGaLinePattern */
static char *uXrtGaLinePattern[] = {
	"LpatNone",
	"LpatSolid",
	"LpatLongDash",
	"LpatDotted",
	"LpatShortDash",
	"LpatLslDash",
	"LpatDashDot"
};

static unsigned char xXrtGaLinePattern[] = {
	XRTGA_LPAT_NONE,
	XRTGA_LPAT_SOLID,
	XRTGA_LPAT_LONG_DASH,
	XRTGA_LPAT_DOTTED,
	XRTGA_LPAT_SHORT_DASH,
	XRTGA_LPAT_LSL_DASH,
	XRTGA_LPAT_DASH_DOT
};

static char *dXrtGaLinePattern[] = {
	"XRTGA_LPAT_NONE",
	"XRTGA_LPAT_SOLID",
	"XRTGA_LPAT_LONG_DASH",
	"XRTGA_LPAT_DOTTED",
	"XRTGA_LPAT_SHORT_DASH",
	"XRTGA_LPAT_LSL_DASH",
	"XRTGA_LPAT_DASH_DOT"
};

/* XrtGaFillPattern */
static char *uXrtGaFillPattern[] = {
	"FpatNone",
	"FpatSolid",
	"Fpat25Percent",
	"Fpat50Percent",
	"Fpat75Percent",
	"FpatHorizStripe",
	"FpatVertStripe",
	"Fpat45Stripe",
	"Fpat135Stripe",
	"FpatDiagHatched",
	"FpatCrossHatched"
};

static unsigned char xXrtGaFillPattern[] = {
	XRTGA_FPAT_NONE,
	XRTGA_FPAT_SOLID,
	XRTGA_FPAT_25_PERCENT,
	XRTGA_FPAT_50_PERCENT,
	XRTGA_FPAT_75_PERCENT,
	XRTGA_FPAT_HORIZ_STRIPE,
	XRTGA_FPAT_VERT_STRIPE,
	XRTGA_FPAT_45_STRIPE,
	XRTGA_FPAT_135_STRIPE,
	XRTGA_FPAT_DIAG_HATCHED,
	XRTGA_FPAT_CROSS_HATCHED
};

static char *dXrtGaFillPattern[] = {
	"XRTGA_FPAT_NONE",
	"XRTGA_FPAT_SOLID",
	"XRTGA_FPAT_25_PERCENT",
	"XRTGA_FPAT_50_PERCENT",
	"XRTGA_FPAT_75_PERCENT",
	"XRTGA_FPAT_HORIZ_STRIPE",
	"XRTGA_FPAT_VERT_STRIPE",
	"XRTGA_FPAT_45_STRIPE",
	"XRTGA_FPAT_135_STRIPE",
	"XRTGA_FPAT_DIAG_HATCHED",
	"XRTGA_FPAT_CROSS_HATCHED"
};

/* XrtGaOffsetType */
static char *uXrtGaOffsetType[] = {
	"OffsetTypeAutomatic",
	"OffsetTypeCircular",
	"OffsetTypeHorizontal",
	"OffsetTypeVertical"
};

static unsigned char xXrtGaOffsetType[] = {
	XRTGA_OFFSET_TYPE_AUTOMATIC,
	XRTGA_OFFSET_TYPE_CIRCULAR,
	XRTGA_OFFSET_TYPE_HORIZONTAL,
	XRTGA_OFFSET_TYPE_VERTICAL
};

static char *dXrtGaOffsetType[] = {
	"XRTGA_OFFSET_TYPE_AUTOMATIC",
	"XRTGA_OFFSET_TYPE_CIRCULAR",
	"XRTGA_OFFSET_TYPE_HORIZONTAL",
	"XRTGA_OFFSET_TYPE_VERTICAL"
};

/* XrtGaNeedleType */
static char *uXrtGaNeedleType[] = {
	"NeedleTypePointer",
	"NeedleTypeTriangle",
	"NeedleTypeArrow"
};

static unsigned char xXrtGaNeedleType[] = {
	XRTGA_NEEDLE_TYPE_POINTER,
	XRTGA_NEEDLE_TYPE_TRIANGLE,
	XRTGA_NEEDLE_TYPE_ARROW
};

static char *dXrtGaNeedleType[] = {
	"XRTGA_NEEDLE_TYPE_POINTER",
	"XRTGA_NEEDLE_TYPE_TRIANGLE",
	"XRTGA_NEEDLE_TYPE_ARROW"
};

/* XrtGaMarkerType */
static char *uXrtGaMarkerType[] = {
	"MarkerTypeCircle",
	"MarkerTypeTriangle",
	"MarkerTypeArrow",
	"MarkerTypeImage"
};

static unsigned char xXrtGaMarkerType[] = {
	XRTGA_MARKER_TYPE_CIRCLE,
	XRTGA_MARKER_TYPE_TRIANGLE,
	XRTGA_MARKER_TYPE_ARROW,
	XRTGA_MARKER_TYPE_IMAGE
};

static char *dXrtGaMarkerType[] = {
	"XRTGA_MARKER_TYPE_CIRCLE",
	"XRTGA_MARKER_TYPE_TRIANGLE",
	"XRTGA_MARKER_TYPE_ARROW",
	"XRTGA_MARKER_TYPE_IMAGE"
};

/* XrtGaShape */
static char *uXrtGaShape[] = {
	"ShapeRectangle",
	"ShapeCircle",
	"ShapeThermometer",
	"ShapeTank",
	"ShapeImage"
};

static unsigned char xXrtGaShape[] = {
	XRTGA_SHAPE_RECTANGLE,
	XRTGA_SHAPE_CIRCLE,
	XRTGA_SHAPE_THERMOMETER,
	XRTGA_SHAPE_TANK,
	XRTGA_SHAPE_IMAGE
};

static char *dXrtGaShape[] = {
	"XRTGA_SHAPE_RECTANGLE",
	"XRTGA_SHAPE_CIRCLE",
	"XRTGA_SHAPE_THERMOMETER",
	"XRTGA_SHAPE_TANK",
	"XRTGA_SHAPE_IMAGE"
};

/* XrtGaSizingPolicy */
static char *uXrtGaSizingPolicy[] = {
	"SizingPolicyWidget",
	"SizingPolicyIcon",
	"SizingPolicyByPosition",
	"SizingPolicyCenter"
};

static unsigned char xXrtGaSizingPolicy[] = {
	XRTGA_SIZING_POLICY_WIDGET,
	XRTGA_SIZING_POLICY_ICON,
	XRTGA_SIZING_POLICY_BY_POSITION,
	XRTGA_SIZING_POLICY_CENTER
};

static char *dXrtGaSizingPolicy[] = {
	"XRTGA_SIZING_POLICY_WIDGET",
	"XRTGA_SIZING_POLICY_ICON",
	"XRTGA_SIZING_POLICY_BY_POSITION",
	"XRTGA_SIZING_POLICY_CENTER"
};

/* XrtGaSwitchType */
static char *uXrtGaSwitchType[] = {
	"SwitchTypeCircle",
	"SwitchTypePointer",
	"SwitchTypeArrow",
	"SwitchTypeFaucet",
	"SwitchTypeImage"
};

static unsigned char xXrtGaSwitchType[] = {
	XRTGA_SWITCH_TYPE_CIRCLE,
	XRTGA_SWITCH_TYPE_POINTER,
	XRTGA_SWITCH_TYPE_ARROW,
	XRTGA_SWITCH_TYPE_FAUCET,
	XRTGA_SWITCH_TYPE_IMAGE
};

static char *dXrtGaSwitchType[] = {
	"XRTGA_SWITCH_TYPE_CIRCLE",
	"XRTGA_SWITCH_TYPE_POINTER",
	"XRTGA_SWITCH_TYPE_ARROW",
	"XRTGA_SWITCH_TYPE_FAUCET",
	"XRTGA_SWITCH_TYPE_IMAGE"
};

/* XrtGaTickType */
static char *uXrtGaTickType[] = {
	"TickTypeRectangle",
	"TickTypeCircle",
	"TickTypeDiamond",
	"TickTypeTriangle"
};

static unsigned char xXrtGaTickType[] = {
	XRTGA_TICK_TYPE_RECTANGLE,
	XRTGA_TICK_TYPE_CIRCLE,
	XRTGA_TICK_TYPE_DIAMOND,
	XRTGA_TICK_TYPE_TRIANGLE
};

static char *dXrtGaTickType[] = {
	"XRTGA_TICK_TYPE_RECTANGLE",
	"XRTGA_TICK_TYPE_CIRCLE",
	"XRTGA_TICK_TYPE_DIAMOND",
	"XRTGA_TICK_TYPE_TRIANGLE"
};

/* XrtGaValueType */
static char *uXrtGaValueType[] = {
	"ValueTypeNumeric",
	"ValueTypeAlphanumeric"
};

static unsigned char xXrtGaValueType[] = {
	XRTGA_VALUE_TYPE_NUMERIC,
	XRTGA_VALUE_TYPE_ALPHANUMERIC
};

static char *dXrtGaValueType[] = {
	"XRTGA_VALUE_TYPE_NUMERIC",
	"XRTGA_VALUE_TYPE_ALPHANUMERIC"
};

/* XrtGaStringMode */
static char *uXrtGaStringMode[] = {
	"StringModeString",
	"StringModeXmString"
};

static unsigned char xXrtGaStringMode[] = {
	XRTGA_STRING_MODE_STRING,
	XRTGA_STRING_MODE_XM_STRING
};

static char *dXrtGaStringMode[] = {
	"XRTGA_STRING_MODE_STRING",
	"XRTGA_STRING_MODE_XM_STRING"
};




#ifndef   RUNTIME

/*
 * ValuesOf functions for both enumerated types and other Xtypes
 */

int
UxValuesOfXrtGaAlignment(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaAlignment;
	*n = XtNumber(uXrtGaAlignment);
	return(*n);
}

int
UxValuesOfXrtGaBorder(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaBorder;
	*n = XtNumber(uXrtGaBorder);
	return(*n);
}

int
UxValuesOfXrtGaCalcMode(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaCalcMode;
	*n = XtNumber(uXrtGaCalcMode);
	return(*n);
}

int
UxValuesOfXrtGaDirection(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaDirection;
	*n = XtNumber(uXrtGaDirection);
	return(*n);
}

int
UxValuesOfXrtGaDisplayMode(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaDisplayMode;
	*n = XtNumber(uXrtGaDisplayMode);
	return(*n);
}

int
UxValuesOfXrtGaDouble(s, n)
char ***s;
int *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_GADOUBLE);
	*s = values;
	*n = 1;

	return 0;
}

int
UxValuesOfXrtGaImage(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_GAIMAGE);
	*s = values;
	*n = 1;
	return 0;
}


int
UxValuesOfXrtGaImageLayout(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaImageLayout;
	*n = XtNumber(uXrtGaImageLayout);
	return(*n);
}

int
UxValuesOfXrtGaIndicatorType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaIndicatorType;
	*n = XtNumber(uXrtGaIndicatorType);
	return(*n);
}

int
UxValuesOfXrtGaInteractionType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaInteractionType;
	*n = XtNumber(uXrtGaInteractionType);
	return(*n);
}

int
UxValuesOfXrtGaLinePattern(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaLinePattern;
	*n = XtNumber(uXrtGaLinePattern);
	return(*n);
}

int
UxValuesOfXrtGaMarkerType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaMarkerType;
	*n = XtNumber(uXrtGaMarkerType);
	return(*n);
}

int
UxValuesOfXrtGaNeedleType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaNeedleType;
	*n = XtNumber(uXrtGaNeedleType);
	return(*n);
}

int
UxValuesOfXrtGaOffsetType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaOffsetType;
	*n = XtNumber(uXrtGaOffsetType);
	return(*n);
}

int
UxValuesOfXrtGaShape(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaShape;
	*n = XtNumber(uXrtGaShape);
	return(*n);
}

int
UxValuesOfXrtGaSizingPolicy(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaSizingPolicy;
	*n = XtNumber(uXrtGaSizingPolicy);
	return(*n);
}

int
UxValuesOfXrtGaStringMode(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaStringMode;
	*n = XtNumber(uXrtGaStringMode);
	return(*n);
}

int
UxValuesOfXrtGaSwitchType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaSwitchType;
	*n = XtNumber(uXrtGaSwitchType);
	return(*n);
}

int
UxValuesOfXrtGaTickType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaTickType;
	*n = XtNumber(uXrtGaTickType);
	return(*n);
}

int
UxValuesOfXrtGaValueType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGaValueType;
	*n = XtNumber(uXrtGaValueType);
	return(*n);
}



/*
 * Validate functions for both enumerated types and other Xtypes.
 * Use a general function for enumerated types.
 */

static int
ValidateXrtGaugeEnumType(rs, s, string_list, n)
swidget rs;
char	*s;
char	**string_list;
int		n;
{
	int i;

	if (s == NULL) {
		return ERROR;
	}
	for (i = 0; i < n; i++) {
		if (UxStrEqual(s, string_list[i])) {
			return NO_ERROR;
		}
	}
	return ERROR;
}


int
UxValidateXrtGaAlignment(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaAlignment,
								  XtNumber(uXrtGaAlignment)));
}

int
UxValidateXrtGaBorder(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaBorder,
								  XtNumber(uXrtGaBorder)));
}

int
UxValidateXrtGaCalcMode(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaCalcMode,
								  XtNumber(uXrtGaCalcMode)));
}

int
UxValidateXrtGaDirection(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaDirection,
								  XtNumber(uXrtGaDirection)));
}

int
UxValidateXrtGaDisplayMode(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaDisplayMode,
								  XtNumber(uXrtGaDisplayMode)));
}

int
UxValidateXrtGaImageLayout(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaImageLayout,
								  XtNumber(uXrtGaImageLayout)));
}

int
UxValidateXrtGaIndicatorType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaIndicatorType,
								  XtNumber(uXrtGaIndicatorType)));
}

int
UxValidateXrtGaInteractionType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaInteractionType,
								  XtNumber(uXrtGaInteractionType)));
}

int
UxValidateXrtGaLinePattern(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaLinePattern,
								  XtNumber(uXrtGaLinePattern)));
}

int
UxValidateXrtGaMarkerType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaMarkerType,
								  XtNumber(uXrtGaMarkerType)));
}

int
UxValidateXrtGaNeedleType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaNeedleType,
								  XtNumber(uXrtGaNeedleType)));
}

int
UxValidateXrtGaOffsetType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaOffsetType,
								  XtNumber(uXrtGaOffsetType)));
}

int
UxValidateXrtGaShape(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaShape,
								  XtNumber(uXrtGaShape)));
}

int
UxValidateXrtGaSizingPolicy(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaSizingPolicy,
								  XtNumber(uXrtGaSizingPolicy)));
}

int
UxValidateXrtGaStringMode(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaStringMode,
								  XtNumber(uXrtGaStringMode)));
}

int
UxValidateXrtGaSwitchType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaSwitchType,
								  XtNumber(uXrtGaSwitchType)));
}

int
UxValidateXrtGaTickType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaTickType,
								  XtNumber(uXrtGaTickType)));
}

int
UxValidateXrtGaValueType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGaugeEnumType(rs, s, uXrtGaValueType,
								  XtNumber(uXrtGaValueType)));
}


#endif   /* ! RUNTIME */

/*
 *	Conversion functions TO and FROM UIMX
 */


static int
String_Image(swgt, string, image, flag, xtype)
swidget			swgt;
char			**string;
XrtGaImage		*image;
int				flag;
int				xtype;
{
	int			i;
	int			status;
	XrmValue	from, to;

	xrt_debug_enter("String_Image");

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 *  Turn a Image into a string.
		 */
		
		xrt_debug_print_string("flag", "TO_UIMX");

		from.size = sizeof(XrtGaImage);
		from.addr = (caddr_t) image;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XmRXrtGaImage, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			xrt_debug_print_string("Output string:", *string);
			xrt_debug_print_int("actual string length", strlen(*string));
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn string into a Image.
		 */
		
		xrt_debug_print_string("flag", "FROM_UIMX");
		xrt_debug_print_string("Input string:", *string);
		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(XrtGaImage);
		to.addr = (caddr_t) image;

		if (XtConvertAndStore (UxTopLevel,
							   XtRString, &from,
							   XmRXrtGaImage, &to))
		{
			status = NO_ERROR;
		}
	}

	xrt_debug_print_int("status", status);
	xrt_debug_leave("String_Image");
	return(status);
}

static int
String_Object(swgt, name, o, flag, xtype)
swidget				swgt;
char				**name;
XrtGaObject			*o;
int					flag;
int					xtype;
{
	swidget			named_swgt;
	int				status;
	XrmValue        from, to;
	XrtStringInfo	*info;

	status = NO_ERROR;

	xrt_debug_enter("String_Object");

	if (flag == TO_UIMX) {
		/*
		 *  object -> name.
		 */
		xrt_debug_print_string("flag", "TO_UIMX");

		if(*o != NULL)
			*name = XtName(*o);
		else
			*name = "";

		xrt_debug_print_string("Output name", *name);

	}
	else {
		/*
		 *  name -> object
		 */

		xrt_debug_print_string("flag", "FROM_UIMX");
		xrt_debug_print_string("Input string:", *name);
		
		named_swgt = UxNameToSwidget(swgt, *name);
		if ( named_swgt == NULL )
			*o = NULL;
		else
			*o = UxGetWidget(named_swgt);

		if (*o == NULL)
			status = ERROR;
	}
	xrt_debug_print_int("status", status);
	xrt_debug_leave("String_Object");
	return(status);
}

static int
String_ObjectList(swgt, string, list, flag, xtype)
swidget			swgt;
char			**string;
XrtGaList		*list;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;

	xrt_debug_enter("String_List");

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 * XrtGaList -> string
		 */
		xrt_debug_print_string("flag", "TO_UIMX");
		
		from.size = sizeof(XrtGaList);
		from.addr = (caddr_t) list;

		to.addr = NULL;

		if (XtConvertAndStore(UxTopLevel,
							  XmRXrtGaObjectList, &from,
							  XmRString, &to))
		{
 			*string = to.addr;
			xrt_debug_print_string("String_List() TO_UIMX: ", *string);
			status = NO_ERROR;
		}
	}
	else {
		/*
		 * string -> XrtGaList
		 * All resources of this type are read only. It means we only
		 * need reverse converter.
		 */

		xrt_debug_print_string("flag", "FROM_UIMX");
		xrt_debug_print_string("Input string:", *string);
		status = NO_ERROR;
	}

done:
	xrt_debug_print_int("status", status);
	xrt_debug_leave("String_List");
	
	return(status);
}

#endif /* HASXRTGAUGE */

void
XrtGauge_enum()
{
#ifdef HASXRTGAUGE
	
	xrt_debug_enter("XrtGauge_enum");

	UxXT_XrtGaAlignment =
		UxAddEnumType(XmRXrtGaAlignment, 
  					  sizeof(unsigned char),
					  xXrtGaAlignment, 
					  uXrtGaAlignment, 
					  dXrtGaAlignment, 
					  XtNumber(uXrtGaAlignment), 
					  UxStringToCharEnum); 
	UxXT_XrtGaBorder =
		UxAddEnumType(XmRXrtGaBorder, 
  					  sizeof(unsigned char),
					  xXrtGaBorder, 
					  uXrtGaBorder, 
					  dXrtGaBorder, 
					  XtNumber(uXrtGaBorder), 
					  UxStringToCharEnum); 
	UxXT_XrtGaCalcMode =
		UxAddEnumType(XmRXrtGaCalcMode, 
  					  sizeof(unsigned char),
					  xXrtGaCalcMode, 
					  uXrtGaCalcMode, 
					  dXrtGaCalcMode, 
					  XtNumber(uXrtGaCalcMode), 
					  UxStringToCharEnum); 
	UxXT_XrtGaDirection =
		UxAddEnumType(XmRXrtGaDirection, 
  					  sizeof(unsigned char),
					  xXrtGaDirection, 
					  uXrtGaDirection, 
					  dXrtGaDirection, 
					  XtNumber(uXrtGaDirection), 
					  UxStringToCharEnum); 
	UxXT_XrtGaDisplayMode =
		UxAddEnumType(XmRXrtGaDisplayMode, 
  					  sizeof(unsigned char),
					  xXrtGaDisplayMode, 
					  uXrtGaDisplayMode, 
					  dXrtGaDisplayMode, 
					  XtNumber(uXrtGaDisplayMode), 
					  UxStringToCharEnum); 
	UxXT_XrtGaImageLayout =
		UxAddEnumType(XmRXrtGaImageLayout, 
  					  sizeof(unsigned char),
					  xXrtGaImageLayout, 
					  uXrtGaImageLayout, 
					  dXrtGaImageLayout, 
					  XtNumber(uXrtGaImageLayout), 
					  UxStringToCharEnum); 
	UxXT_XrtGaIndicatorType =
		UxAddEnumType(XmRXrtGaIndicatorType, 
  					  sizeof(unsigned char),
					  xXrtGaIndicatorType, 
					  uXrtGaIndicatorType, 
					  dXrtGaIndicatorType, 
					  XtNumber(uXrtGaIndicatorType), 
					  UxStringToCharEnum); 
	UxXT_XrtGaInteractionType =
		UxAddEnumType(XmRXrtGaInteractionType, 
  					  sizeof(unsigned char),
					  xXrtGaInteractionType, 
					  uXrtGaInteractionType, 
					  dXrtGaInteractionType, 
					  XtNumber(uXrtGaInteractionType), 
					  UxStringToCharEnum); 
	UxXT_XrtGaLinePattern =
		UxAddEnumType(XmRXrtGaLinePattern, 
  					  sizeof(unsigned char),
					  xXrtGaLinePattern, 
					  uXrtGaLinePattern, 
					  dXrtGaLinePattern, 
					  XtNumber(uXrtGaLinePattern), 
					  UxStringToCharEnum); 
	UxXT_XrtGaMarkerType =
		UxAddEnumType(XmRXrtGaMarkerType, 
  					  sizeof(unsigned char),
					  xXrtGaMarkerType, 
					  uXrtGaMarkerType, 
					  dXrtGaMarkerType, 
					  XtNumber(uXrtGaMarkerType), 
					  UxStringToCharEnum); 
	UxXT_XrtGaNeedleType =
		UxAddEnumType(XmRXrtGaNeedleType, 
  					  sizeof(unsigned char),
					  xXrtGaNeedleType, 
					  uXrtGaNeedleType, 
					  dXrtGaNeedleType, 
					  XtNumber(uXrtGaNeedleType), 
					  UxStringToCharEnum); 
	UxXT_XrtGaOffsetType =
		UxAddEnumType(XmRXrtGaOffsetType, 
  					  sizeof(unsigned char),
					  xXrtGaOffsetType, 
					  uXrtGaOffsetType, 
					  dXrtGaOffsetType, 
					  XtNumber(uXrtGaOffsetType), 
					  UxStringToCharEnum); 
	UxXT_XrtGaShape =
		UxAddEnumType(XmRXrtGaShape, 
  					  sizeof(unsigned char),
					  xXrtGaShape, 
					  uXrtGaShape, 
					  dXrtGaShape, 
					  XtNumber(uXrtGaShape), 
					  UxStringToCharEnum);
	UxXT_XrtGaSizingPolicy =
		UxAddEnumType(XmRXrtGaSizingPolicy, 
  					  sizeof(unsigned char),
					  xXrtGaSizingPolicy, 
					  uXrtGaSizingPolicy, 
					  dXrtGaSizingPolicy, 
					  XtNumber(uXrtGaSizingPolicy), 
					  UxStringToCharEnum); 
	UxXT_XrtGaStringMode =
		UxAddEnumType(XmRXrtGaStringMode, 
  					  sizeof(unsigned char),
					  xXrtGaStringMode, 
					  uXrtGaStringMode, 
					  dXrtGaStringMode, 
					  XtNumber(uXrtGaStringMode), 
					  UxStringToCharEnum); 
	UxXT_XrtGaSwitchType =
		UxAddEnumType(XmRXrtGaSwitchType, 
  					  sizeof(unsigned char),
					  xXrtGaSwitchType, 
					  uXrtGaSwitchType, 
					  dXrtGaSwitchType, 
					  XtNumber(uXrtGaSwitchType), 
					  UxStringToCharEnum); 
	UxXT_XrtGaTickType =
		UxAddEnumType(XmRXrtGaTickType, 
  					  sizeof(unsigned char),
					  xXrtGaTickType, 
					  uXrtGaTickType, 
					  dXrtGaTickType, 
					  XtNumber(uXrtGaTickType), 
					  UxStringToCharEnum); 
	UxXT_XrtGaValueType =
		UxAddEnumType(XmRXrtGaValueType, 
  					  sizeof(unsigned char),
					  xXrtGaValueType, 
					  uXrtGaValueType, 
					  dXrtGaValueType, 
					  XtNumber(uXrtGaValueType), 
					  UxStringToCharEnum); 

#endif /*HASXRTGAUGE*/
}

/* Add new XRT/gauge xtypes */
void
XrtGauge_xtypes()
{
#ifdef HASXRTGAUGE

	xrt_debug_enter("XrtGauge_xtypes");

	/*
	 * Initialize widget class in order to initialize converters.  Also
	 * install reverse converters.
	 */
	XtInitializeWidgetClass(xmXrtGaCircularDialWidgetClass);
	XtInitializeWidgetClass(xmXrtGaCircularGaugeWidgetClass);
	XtInitializeWidgetClass(xmXrtGaLEDIconWidgetClass);
	XtInitializeWidgetClass(xmXrtGaLEDNumericalWidgetClass);
	XtInitializeWidgetClass(xmXrtGaLinearDialWidgetClass);
	XtInitializeWidgetClass(xmXrtGaLinearGaugeWidgetClass);
	XtInitializeWidgetClass(xmXrtGaPercentWidgetClass);

	UxXT_XrtGaDouble		= UxAddXtype(XmRXrtGaDouble, sizeof(double));
	UxXT_XrtGaImage			= UxAddXtype(XmRXrtGaImage,sizeof(XrtGaImage));
	UxXT_XrtGaObject		= UxAddXtype(XmRXrtGaObject,sizeof(XrtGaObject));
	UxXT_XrtGaObjectList	= UxAddXtype(XmRXrtGaObjectList,sizeof(XrtGaList));

	/* Add new converters for Xtypes */
  	UxAddConv(UxUT_string, UxXT_XrtGaImage,		String_Image);
  	UxAddConv(UxUT_string, UxXT_XrtGaObject,	String_Object);
  	UxAddConv(UxUT_string, UxXT_XrtGaObjectList,String_ObjectList);
	xrt_debug_leave("XrtGauge_xtypes");

#endif /* HASXRTGAUGE */
}

#if HASXRTGAUGE
/*
 * Special get function to replace UxGetProp 
 * for resources of type "double". UxGetProp and UxDDGetProp
 * always return XtArgVal but XRT/gauge is somewhat asymetric
 * when it comes to putting and getting doubles. This funtion
 * gets the double (XRT style) then returns a pointer to it
 * which the UxGet... macro dereferences using XrtGaArgValToDouble
 */

#if FALSE
double
XrtGaArgValToDouble(v)
XtArgVal v;
{
    return(*(double *)v);
}
#endif

XtArgVal
UxGetXrtGaDoubleProp(sw, res)
swidget	sw;
char	*res;
{
	Widget      	w;
	static double	value = (double) 0.0;

	xrt_debug_enter("XrtGaGetDoubleProp");

	if (UxIsSwidget(sw) && (w = (Widget) UxGetWidget(sw)) != NULL) {
		XtVaGetValues (w, res, &value, NULL);

		xrt_debug_leave("XrtGaGetDoubleProp");
		return (XtArgVal)&value;
	}
}
#endif /* HASXRTGAUGE */
