/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
 *
 * XRT/gear widget UIM/X integration file
 *
 *  SCCSID -- @(#)XrtGear_xtype.c	1.12    99/01/29 KL Group Inc.
 */

/******************************************************************************
				XrtGear_xtype.c
******************************************************************************/

#include <UxLib.h>
#include <vtypes.h>
#include <utype.h>
#include <ctype.h>
#include <string.h>
#include <time.h>

#ifdef DESIGN_TIME

#include "veos.h"       		/* Visual Edge object system */
#include "valuesOf.h"   		/* ValuesOf & Validator functions */
#include "resource.h"
#include "resvalue.h"
#include "plist.cl.h"
#include "observer.h"
#include "swidget.h"

#endif /* DESIGN_TIME */

#include "Xrt_restable.h"

#include <Xm/XrtGear.h>

/*temporary include all .h files*/
#include <Xm/XrtGearAll.h>

#include <cat_macros.h>

/* Begin new XRT/gear types */

int		UxXT_XrtGearArmMask;			
int		UxXT_XrtGearArmPixmap; 			
int		UxXT_XrtGearAlignerAlignment;
int		UxXT_XrtGearAlignment;
int		UxXT_XrtGearAnchorSide;
int		UxXT_XrtGearBorderType;
int		UxXT_XrtGearChildType;
int		UxXT_XrtGearIndicator;
int		UxXT_XrtGearIndicatorPixmapList;
int		UxXT_XrtGearIndicatorMaskList;
int		UxXT_XrtGearMask;
int		UxXT_XrtGearPageAttachPolicy;		
int		UxXT_XrtGearPixmap; 			
int		UxXT_XrtGearRotation;
int		UxXT_XrtGearStringLayout;
int		UxXT_XrtGearStyle;
int		UxXT_XrtGearTabSide;
int		UxXT_XrtGearType;
int		UxXT_XrtGearWidget;

/*gear2.0*/
int		UxXT_XrtGearArrowType;
int		UxXT_XrtGearBorderLocation;
int		UxXT_XrtGearColumn;
int		UxXT_XrtGearColumnResizePolicy;
int		UxXT_XrtGearCursor;
int		UxXT_XrtGearDisplayPolicy;
int		UxXT_XrtGearFolderStateMask;
int		UxXT_XrtGearIcon;
int		UxXT_XrtGearImportPolicy;
int		UxXT_XrtGearIndentPolicy;
int		UxXT_XrtGearLineStyle;
int		UxXT_XrtGearMenuAlignment;
int		UxXT_XrtGearNodeMovePolicy;
int		UxXT_XrtGearNodeStyle;
int		UxXT_XrtGearProgressType;
int     UxXT_XrtGearPSFontMapList;
int		UxXT_XrtGearSelectionPolicy;
int		UxXT_XrtGearSelectionRestrictions;
int		UxXT_XrtGearSearchPolicy;
int		UxXT_XrtGearSizingPolicy;
int		UxXT_XrtGearString;
int		UxXT_XrtGearStringTable;
int		UxXT_XrtGearTextClipPolicy;
int		UxXT_XrtGearUlong;
int		UxXT_XrtGearWidgetResizePolicy;
int		UxXT_XrtGearEditType;

/* Define three static arrays for each enumerated type */

/* XrtGearAlignerAlignment  */
static char *uXrtGearAlignerAlignment[] = {
    "Aligner_Beginning",
    "Aligner_Center",
    "Aligner_End",
	"Aligner_Topright",
	"Aligner_Middleright",
	"Aligner_Bottomright",
	"Aligner_Topcenter",
	"Aligner_Middlecenter",
	"Aligner_Bottomcenter",
	"Aligner_Topleft",
	"Aligner_Middleleft",
	"Aligner_Bottomleft"
};
static unsigned char xXrtGearAlignerAlignment[] = {
    XRTGEAR_ALIGNER_BEGINNING,
    XRTGEAR_ALIGNER_CENTER,
    XRTGEAR_ALIGNER_END,
	XRTGEAR_ALIGNER_TOPRIGHT,
	XRTGEAR_ALIGNER_MIDDLERIGHT,
	XRTGEAR_ALIGNER_BOTTOMRIGHT,
	XRTGEAR_ALIGNER_TOPCENTER,
	XRTGEAR_ALIGNER_MIDDLECENTER,
	XRTGEAR_ALIGNER_BOTTOMCENTER,
	XRTGEAR_ALIGNER_TOPLEFT,
	XRTGEAR_ALIGNER_MIDDLELEFT,
	XRTGEAR_ALIGNER_BOTTOMLEFT
};
static char	*dXrtGearAlignerAlignment[] = {
    "XRTGEAR_ALIGNER_BEGINNING",
    "XRTGEAR_ALIGNER_CENTER",
    "XRTGEAR_ALIGNER_END",
	"XRTGEAR_ALIGNER_TOPRIGHT",
	"XRTGEAR_ALIGNER_MIDDLERIGHT",
	"XRTGEAR_ALIGNER_BOTTOMRIGHT",
	"XRTGEAR_ALIGNER_TOPCENTER",
	"XRTGEAR_ALIGNER_MIDDLECENTER",
	"XRTGEAR_ALIGNER_BOTTOMCENTER",
	"XRTGEAR_ALIGNER_TOPLEFT",
	"XRTGEAR_ALIGNER_MIDDLELEFT",
	"XRTGEAR_ALIGNER_BOTTOMLEFT"
};

/* XrtGearAlignment  */
static char *uXrtGearAlignment[] = {
    "Align_Beginning",
    "Align_Center",
    "Align_End"
};
static unsigned char xXrtGearAlignment[] = {
    XRTGEAR_ALIGN_BEGINNING,
    XRTGEAR_ALIGN_CENTER,
    XRTGEAR_ALIGN_END
};
static char	*dXrtGearAlignment[] = {
    "XRTGEAR_ALIGN_BEGINNING",
    "XRTGEAR_ALIGN_CENTER",
    "XRTGEAR_ALIGN_END"
};

/* XrtGearAnchorSide */
static char *uXrtGearAnchorSide[] = {
	"Anchor_Side_Left",
	"Anchor_Side_Right",
	"Anchor_Side_Top",
	"Anchor_Side_Bottom"
};
static unsigned char xXrtGearAnchorSide[] = {
	XRTGEAR_ANCHOR_SIDE_LEFT,
	XRTGEAR_ANCHOR_SIDE_RIGHT,
	XRTGEAR_ANCHOR_SIDE_TOP,
	XRTGEAR_ANCHOR_SIDE_BOTTOM
};
static char	*dXrtGearAnchorSide[] = {
	"XRTGEAR_ANCHOR_SIDE_LEFT",
	"XRTGEAR_ANCHOR_SIDE_RIGHT",
	"XRTGEAR_ANCHOR_SIDE_TOP",
	"XRTGEAR_ANCHOR_SIDE_BOTTOM"
};

/* XrtGearBorderType  */
static char *uXrtGearBorderType[] = {
    "Border_None",
    "Border_Etched_In",
    "Border_Etched_Out",
    "Border_In",
    "Border_Out",
    "Border_Frame_In",
    "Border_Frame_Out",
    "Border_Bevel",
    "Border_Shadow",
    "Border_Plain"
};
static unsigned char xXrtGearBorderType[] = {
    XRTGEAR_BORDER_NONE,
    XRTGEAR_BORDER_ETCHED_IN,
    XRTGEAR_BORDER_ETCHED_OUT,
    XRTGEAR_BORDER_IN,
    XRTGEAR_BORDER_OUT,
    XRTGEAR_BORDER_FRAME_IN,
    XRTGEAR_BORDER_FRAME_OUT,
    XRTGEAR_BORDER_BEVEL,
    XRTGEAR_BORDER_SHADOW,
    XRTGEAR_BORDER_PLAIN
};
static char	*dXrtGearBorderType[] = {
    "XRTGEAR_BORDER_NONE",
    "XRTGEAR_BORDER_ETCHED_IN",
    "XRTGEAR_BORDER_ETCHED_OUT",
    "XRTGEAR_BORDER_IN",
    "XRTGEAR_BORDER_OUT",
    "XRTGEAR_BORDER_FRAME_IN",
    "XRTGEAR_BORDER_FRAME_OUT",
    "XRTGEAR_BORDER_BEVEL",
    "XRTGEAR_BORDER_SHADOW",
    "XRTGEAR_BORDER_PLAIN"
};

/* XrtGearChildType  */
static char *uXrtGearChildType[] = {
	"Child_Type_Page",
	"Child_Type_Tab",
	"Child_Type_Internal"
};
static unsigned char xXrtGearChildType[] = {
	XRTGEAR_CHILD_TYPE_PAGE,
	XRTGEAR_CHILD_TYPE_TAB,
	XRTGEAR_CHILD_TYPE_INTERNAL
};
static char	*dXrtGearChildType[] = {
	"XRTGEAR_CHILD_TYPE_PAGE",
	"XRTGEAR_CHILD_TYPE_TAB",
	"XRTGEAR_CHILD_TYPE_INTERNAL"
};

/* XrtGearIndicator  */
static char *uXrtGearIndicator[] = {
	"Indicator_None",
	"Indicator_Fill",
	"Indicator_Check",
	"Indicator_Check_Box",
	"Indicator_Circle",
	"Indicator_Circle_Box",
	"Indicator_Cross",
	"Indicator_Cross_Box",
	"Indicator_Pixmap"
};
static unsigned char xXrtGearIndicator[] = {
	XRTGEAR_INDICATOR_NONE,
	XRTGEAR_INDICATOR_FILL,
	XRTGEAR_INDICATOR_CHECK,
	XRTGEAR_INDICATOR_CHECK_BOX,
	XRTGEAR_INDICATOR_CIRCLE,
	XRTGEAR_INDICATOR_CIRCLE_BOX,
	XRTGEAR_INDICATOR_CROSS,
	XRTGEAR_INDICATOR_CROSS_BOX,
	XRTGEAR_INDICATOR_PIXMAP
};
static char	*dXrtGearIndicator[] = {
	"XRTGEAR_INDICATOR_NONE",
	"XRTGEAR_INDICATOR_FILL",
	"XRTGEAR_INDICATOR_CHECK",
	"XRTGEAR_INDICATOR_CHECK_BOX",
	"XRTGEAR_INDICATOR_CIRCLE",
	"XRTGEAR_INDICATOR_CIRCLE_BOX",
	"XRTGEAR_INDICATOR_CROSS",
	"XRTGEAR_INDICATOR_CROSS_BOX",
	"XRTGEAR_INDICATOR_PIXMAP"
};

/* XrtGearPageAttachPolicy  */
static char *uXrtGearPageAttachPolicy[] = {
	"Attach_Always",
	"Attach_None"
};
static unsigned char xXrtGearPageAttachPolicy[] = {
	XRTGEAR_ATTACH_ALWAYS,
	XRTGEAR_ATTACH_NONE
};
static char	*dXrtGearPageAttachPolicy[] = {
	"XRTGEAR_ATTACH_ALWAYS",
	"XRTGEAR_ATTACH_NONE"
};

/* XrtGearRotation  */
static char *uXrtGearRotation[] = {
	"Rotate_None",
	"Rotate_90",
	"Rotate_180",
	"Rotate_270",
	"Rotate_Unspecified"
};
static unsigned char xXrtGearRotation[] = {
	XRTGEAR_ROTATE_NONE,
	XRTGEAR_ROTATE_90,
	XRTGEAR_ROTATE_180,
	XRTGEAR_ROTATE_270,
	XRTGEAR_ROTATE_UNSPECIFIED
};
static char	*dXrtGearRotation[] = {
	"XRTGEAR_ROTATE_NONE",
	"XRTGEAR_ROTATE_90",
	"XRTGEAR_ROTATE_180",
	"XRTGEAR_ROTATE_270",
	"XRTGEAR_ROTATE_UNSPECIFIED"
};

/* XrtGearTabSide  */
static char *uXrtGearTabSide[] = {
	"Side_Left",
	"Side_Right",
	"Side_Top",
	"Side_Bottom"
};
static unsigned char xXrtGearTabSide[] = {
	XRTGEAR_SIDE_LEFT,
	XRTGEAR_SIDE_RIGHT,
	XRTGEAR_SIDE_TOP,
	XRTGEAR_SIDE_BOTTOM
};
static char	*dXrtGearTabSide[] = {
	"XRTGEAR_SIDE_LEFT",
	"XRTGEAR_SIDE_RIGHT",
	"XRTGEAR_SIDE_TOP",
	"XRTGEAR_SIDE_BOTTOM"
};

/* XrtGearStringLayout  */
static char *uXrtGearStringLayout[] = {
	"String_Center",
	"String_Top",
	"String_Left",
	"String_Bottom",
	"String_Right"
};
static unsigned char xXrtGearStringLayout[] = {
	XRTGEAR_STRING_CENTER,
	XRTGEAR_STRING_TOP,
	XRTGEAR_STRING_LEFT,
	XRTGEAR_STRING_BOTTOM,
	XRTGEAR_STRING_RIGHT
};
static char	*dXrtGearStringLayout[] = {
	"XRTGEAR_STRING_CENTER",
	"XRTGEAR_STRING_TOP",
	"XRTGEAR_STRING_LEFT",
	"XRTGEAR_STRING_BOTTOM",
	"XRTGEAR_STRING_RIGHT"
};
/* XrtGearStyle  */
static char *uXrtGearStyle[] = {
	"Style_Rounded",
	"Style_Square",
	"Style_Slanted",
	"Style_Chamfered"
};
static unsigned char xXrtGearStyle[] = {
	XRTGEAR_STYLE_ROUNDED,
	XRTGEAR_STYLE_SQUARE,
	XRTGEAR_STYLE_SLANTED,
	XRTGEAR_STYLE_CHAMFERED
};
static char	*dXrtGearStyle[] = {
	"XRTGEAR_STYLE_ROUNDED",
	"XRTGEAR_STYLE_SQUARE",
	"XRTGEAR_STYLE_SLANTED",
	"XRTGEAR_STYLE_CHAMFERED"
};

/* XrtGearType  */
static char *uXrtGearType[] = {
	"Type_Unspecified",
	"Type_String",
	"Type_Pixmap",
    "Type_String_Pixmap"
};
static unsigned char xXrtGearType[] = {
	XRTGEAR_TYPE_UNSPECIFIED,
	XRTGEAR_TYPE_STRING,
	XRTGEAR_TYPE_PIXMAP,
    XRTGEAR_TYPE_STRING_PIXMAP
};
static char	*dXrtGearType[] = {
	"XRTGEAR_TYPE_UNSPECIFIED",
	"XRTGEAR_TYPE_STRING",
	"XRTGEAR_TYPE_PIXMAP",
    "XRTGEAR_TYPE_STRING_PIXMAP"
};

/* New enums in Gear 2.0*/

/*  XrtGearArrowType */
static char * uXrtGearArrowType[] = {
	"Arrow_None",
	"Arrow_Wedge",
	"Arrow_With_Underbar",
	"Arrow_Custom"
};
static unsigned char xXrtGearArrowType[] = {
	XRTGEAR_ARROW_NONE,
	XRTGEAR_ARROW_WEDGE,
	XRTGEAR_ARROW_WITH_UNDERBAR,
	XRTGEAR_ARROW_CUSTOM
};
static char * dXrtGearArrowType[] = {
    "XRTGEAR_ARROW_NONE",
	"XRTGEAR_ARROW_WEDGE",
	"XRTGEAR_ARROW_WITH_UNDERBAR",
	"XRTGEAR_ARROW_CUSTOM"
};

/*  XrtGearBorderLocation */
static char * uXrtGearBorderLocation[] = {
  "Border_Loc_Labels",
  "Border_Loc_Nodes",
  "Border_Loc_Labels_Separated_From_Nodes",
  "Border_Loc_Each_Label_Separated_From_Nodes",
  "Border_Loc_Each_Label",
  "Border_Loc_Labels_And_Nodes"
};
static unsigned char xXrtGearBorderLocation[] = {
  XRTGEAR_BORDER_LOC_LABELS,
  XRTGEAR_BORDER_LOC_NODES,
  XRTGEAR_BORDER_LOC_LABELS_SEPARATED_FROM_NODES,
  XRTGEAR_BORDER_LOC_EACH_LABEL_SEPARATED_FROM_NODES,
  XRTGEAR_BORDER_LOC_EACH_LABEL,
  XRTGEAR_BORDER_LOC_LABELS_AND_NODES
};
static char * dXrtGearBorderLocation[] = {
  "XRTGEAR_BORDER_LOC_LABELS",
  "XRTGEAR_BORDER_LOC_NODES",
  "XRTGEAR_BORDER_LOC_LABELS_SEPARATED_FROM_NODES",
  "XRTGEAR_BORDER_LOC_EACH_LABEL_SEPARATED_FROM_NODES",
  "XRTGEAR_BORDER_LOC_EACH_LABEL",
  "XRTGEAR_BORDER_LOC_LABELS_AND_NODES"
};

/* XrtGearColumnResizePolicy */
static char * uXrtGearColumnResizePolicy[] = {
  "Column_Resize_All",
  "Column_Resize_Labels_Only",
  "Column_Resize_None"
};
static unsigned char xXrtGearColumnResizePolicy[] = {
  XRTGEAR_COLUMN_RESIZE_ALL,
  XRTGEAR_COLUMN_RESIZE_LABELS_ONLY,
  XRTGEAR_COLUMN_RESIZE_NONE
};
static char * dXrtGearColumnResizePolicy[] = {
  "XRTGEAR_COLUMN_RESIZE_ALL",
  "XRTGEAR_COLUMN_RESIZE_LABELS_ONLY",
  "XRTGEAR_COLUMN_RESIZE_NONE"
};

/* XrtGearDisplayPolicy */
static char * uXrtGearDisplayPolicy[] = {
  "Display_As_Needed",
  "Display_Always",
  "Display_Never"
};
static unsigned char xXrtGearDisplayPolicy[] = {
  XRTGEAR_DISPLAY_AS_NEEDED,
  XRTGEAR_DISPLAY_ALWAYS,
  XRTGEAR_DISPLAY_NEVER
};
static char * dXrtGearDisplayPolicy[] = {
  "XRTGEAR_DISPLAY_AS_NEEDED",
  "XRTGEAR_DISPLAY_ALWAYS",
  "XRTGEAR_DISPLAY_NEVER"
};

/* XrtGearImportPolicy */
static char * uXrtGearImportPolicy[] = {
	"Import_Nothing",
	"Import_Anything",
	"Import_Within_App",
    "Import_Within_Widget"
};
static unsigned char xXrtGearImportPolicy[] = {
	XRTGEAR_IMPORT_NOTHING,
	XRTGEAR_IMPORT_ANYTHING,
	XRTGEAR_IMPORT_WITHIN_APP,
    XRTGEAR_IMPORT_WITHIN_WIDGET
};
static char * dXrtGearImportPolicy[] = {
	"XRTGEAR_IMPORT_NOTHING",
	"XRTGEAR_IMPORT_ANYTHING",
	"XRTGEAR_IMPORT_WITHIN_APP",
    "XRTGEAR_IMPORT_WITHIN_WIDGET"
};

/* XrtGearIndentPolicy */
static char * uXrtGearIndentPolicy[] = {
	"Indent_Pixel",
	"Indent_Char",
	"Indent_None"
};
static unsigned char xXrtGearIndentPolicy[] = {
	XRTGEAR_INDENT_PIXEL,
	XRTGEAR_INDENT_CHAR,
	XRTGEAR_INDENT_NONE
};
static char * dXrtGearIndentPolicy[] = {
	"XRTGEAR_INDENT_PIXEL",
	"XRTGEAR_INDENT_CHAR",
	"XRTGEAR_INDENT_NONE"
};

/* XrtGearLineStyle */
static char * uXrtGearLineStyle[] = {
  "Linestyle_None",
  "Linestyle_Dotted",
  "Linestyle_Solid",
};
static unsigned char xXrtGearLineStyle[] = {
  XRTGEAR_LINESTYLE_NONE,
  XRTGEAR_LINESTYLE_DOTTED,
  XRTGEAR_LINESTYLE_SOLID,
};
static char * dXrtGearLineStyle[] = {
  "XRTGEAR_LINESTYLE_NONE",
  "XRTGEAR_LINESTYLE_DOTTED",
  "XRTGEAR_LINESTYLE_SOLID",
};

/*  XrtGearMenuAlignment */
static char * uXrtGearMenuAlignment[] = {
	"Menu_Align_Beginning",
	"Menu_Align_Both",
    "Menu_Align_End"
};
static unsigned char xXrtGearMenuAlignment[] = {
	XRTGEAR_MENU_ALIGN_BEGINNING,
	XRTGEAR_MENU_ALIGN_BOTH,
    XRTGEAR_MENU_ALIGN_END
};
static char * dXrtGearMenuAlignment[] = {
	"XRTGEAR_MENU_ALIGN_BEGINNING",
	"XRTGEAR_MENU_ALIGN_BOTH",
    "XRTGEAR_MENU_ALIGN_END"
};

/* XrtGearNodeMovePolicy */
static char * uXrtGearNodeMovePolicy[] = {
  "Move_Nowhere",
  "Move_Within_Level",
  "Move_Within_Widget",
  "Move_Within_App"
};
static unsigned char xXrtGearNodeMovePolicy[] = {
  XRTGEAR_MOVE_NOWHERE,
  XRTGEAR_MOVE_WITHIN_LEVEL,
  XRTGEAR_MOVE_WITHIN_WIDGET,
  XRTGEAR_MOVE_WITHIN_APP
};
static char * dXrtGearNodeMovePolicy[] = {
  "XRTGEAR_MOVE_NOWHERE",
  "XRTGEAR_MOVE_WITHIN_LEVEL",
  "XRTGEAR_MOVE_WITHIN_WIDGET",
  "XRTGEAR_MOVE_WITHIN_APP"
};

/*  XrtGearProgressType */
static char * uXrtGearProgressType[] = {
	"Progress_Bar",
	"Progress_Discrete_Bar",
	"Progress_Icon",
	"Progress_Stacking_Icons"
};
static unsigned char xXrtGearProgressType[] = {
	XRTGEAR_PROGRESS_BAR,
	XRTGEAR_PROGRESS_DISCRETE_BAR,
	XRTGEAR_PROGRESS_ICON,
	XRTGEAR_PROGRESS_STACKING_ICONS
};
static char * dXrtGearProgressType[] = {
	"XRTGEAR_PROGRESS_BAR",
	"XRTGEAR_PROGRESS_DISCRETE_BAR",
	"XRTGEAR_PROGRESS_ICON",
	"XRTGEAR_PROGRESS_STACKING_ICONS"
};

/* XrtGearSelectionPolicy */
static char * uXrtGearSelectionPolicy[] = {
  "Selection_Policy_None",
  "Selection_Policy_Single",
  "Selection_Policy_Single_Auto_Select",
  "Selection_Policy_Range",
  "Selection_Policy_Multiple",
  "Selection_Policy_Multiple"
};
static unsigned char xXrtGearSelectionPolicy[] = {
  XRTGEAR_SELECTION_POLICY_NONE,
  XRTGEAR_SELECTION_POLICY_SINGLE,
  XRTGEAR_SELECTION_POLICY_SINGLE_AUTO_SELECT,
  XRTGEAR_SELECTION_POLICY_RANGE,
  XRTGEAR_SELECTION_POLICY_MULTIPLE,
  XRTGEAR_SELECTION_POLICY_MULTIPLE_AUTO_UNSELECT
};
static char * dXrtGearSelectionPolicy[] = {
  "XRTGEAR_SELECTION_POLICY_NONE",
  "XRTGEAR_SELECTION_POLICY_SINGLE",
  "XRTGEAR_SELECTION_POLICY_SINGLE_AUTO_SELECT",
  "XRTGEAR_SELECTION_POLICY_RANGE",
  "XRTGEAR_SELECTION_POLICY_MULTIPLE",
  "XRTGEAR_SELECTION_POLICY_MULTIPLE"
};

/* XrtGearSearchPolicy */
static char * uXrtGearSearchPolicy[] = {
   "Search_Single",
   "Search_Multichar"
};
static unsigned char xXrtGearSearchPolicy[] = {
    XRTGEAR_SEARCH_SINGLE,
    XRTGEAR_SEARCH_MULTICHAR
};
static char * dXrtGearSearchPolicy[] = {
   "XRTGEAR_SEARCH_SINGLE",
   "XRTGEAR_SEARCH_MULTICHAR"
};

/* XrtGearSizingPolicy */
static char * uXrtGearSizingPolicy[] = {
  "Size_Pixel",
  "Size_Char",
  "Size_Node",
  "Size_Column",
  "Size_Label_Variable",
  "Size_Variable"
};
static unsigned char xXrtGearSizingPolicy[] = {
  XRTGEAR_SIZE_PIXEL,
  XRTGEAR_SIZE_CHAR,
  XRTGEAR_SIZE_NODE,
  XRTGEAR_SIZE_COLUMN,
  XRTGEAR_SIZE_LABEL_VARIABLE,
  XRTGEAR_SIZE_VARIABLE
};
static char * dXrtGearSizingPolicy[] = {
  "XRTGEAR_SIZE_PIXEL",
  "XRTGEAR_SIZE_CHAR",
  "XRTGEAR_SIZE_NODE",
  "XRTGEAR_SIZE_COLUMN",
  "XRTGEAR_SIZE_LABEL_VARIABLE",
  "XRTGEAR_SIZE_VARIABLE"
};

/* XrtGearTextClipPolicy */
static char * uXrtGearTextClipPolicy[] = {
  "Truncate",
  "Truncate_With_Arrow",
  "Compress_With_Ellipsis"
};
static unsigned char xXrtGearTextClipPolicy[] = {
  XRTGEAR_TRUNCATE,
  XRTGEAR_TRUNCATE_WITH_ARROW,
  XRTGEAR_COMPRESS_WITH_ELLIPSIS
};
static char * dXrtGearTextClipPolicy[] = {
  "XRTGEAR_TRUNCATE",
  "XRTGEAR_TRUNCATE_WITH_ARROW",
  "XRTGEAR_COMPRESS_WITH_ELLIPSIS"
};


/* XrtGearWidgetResizePolicy */
static char * uXrtGearWidgetResizePolicy[] = {
  "Widget_Resize_Variable",
  "Widget_Resize_Grow_Only",
  "Widget_Resize_Fixed"
};
static unsigned char xXrtGearWidgetResizePolicy[] = {
  XRTGEAR_WIDGET_RESIZE_VARIABLE,
  XRTGEAR_WIDGET_RESIZE_GROW_ONLY,
  XRTGEAR_WIDGET_RESIZE_FIXED
};
static char * dXrtGearWidgetResizePolicy[] = {
  "XRTGEAR_WIDGET_RESIZE_VARIABLE",
  "XRTGEAR_WIDGET_RESIZE_GROW_ONLY",
  "XRTGEAR_WIDGET_RESIZE_FIXED"
};

static char * uXrtGearEditType[] = {
	"Edit_Type_Windows",
	"Edit_Type_Motif"
};
static unsigned char xXrtGearEditType[] = {
	XRTGEAR_EDIT_TYPE_WINDOWS,
	XRTGEAR_EDIT_TYPE_MOTIF
};
static char * dXrtGearEditType[] = {
	"XRTGEAR_EDIT_TYPE_WINDOWS",
	"XRTGEAR_EDIT_TYPE_MOTIF"
};

#ifndef   RUNTIME

/*
 * ValuesOf functions for both enumerated types and other Xtypes
 */

int
UxValuesOfXrtGearAlignerAlignment(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearAlignerAlignment;
	*n = XtNumber(uXrtGearAlignerAlignment);
	return(*n);
}

int
UxValuesOfXrtGearAlignment(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearAlignment;
	*n = XtNumber(uXrtGearAlignment);
	return(*n);
}

int
UxValuesOfXrtGearAnchorSide(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearAnchorSide;
	*n = XtNumber(uXrtGearAnchorSide);
	return(*n);
}

int
UxValuesOfXrtGearBorderType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearBorderType;
	*n = XtNumber(uXrtGearBorderType);
	return(*n);
}

int
UxValuesOfXrtGearChildType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearChildType;
	*n = XtNumber(uXrtGearChildType);
	return(*n);
}
int
UxValuesOfXrtGearIndicator(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearIndicator;
	*n = XtNumber(uXrtGearIndicator);
	return(*n);
}

int
UxValuesOfXrtGearMask(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0]= CGETS_KL(VALOF_GEARMASK);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtGearIndicatorPixmapList(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_GEARINDICATORPIXMAPLIST);
	*s = values;
	*n = 1;
	return 0;
}
UxValuesOfXrtGearIndicatorMaskList(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_GEARINDICATORMASKLIST);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtGearPageAttachPolicy(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearPageAttachPolicy;
	*n = XtNumber(uXrtGearPageAttachPolicy);
	return(*n);
}

int
UxValuesOfXrtGearPixmap(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0]= CGETS_KL(VALOF_GEARPIXMAP);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtGearRotation(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearRotation;
	*n = XtNumber(uXrtGearRotation);
	return(*n);
}

int
UxValuesOfXrtGearTabSide(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearTabSide;
	*n = XtNumber(uXrtGearTabSide);
	return(*n);
}

int
UxValuesOfXrtGearStringLayout(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearStringLayout;
	*n = XtNumber(uXrtGearStringLayout);
	return(*n);
}

int
UxValuesOfXrtGearStyle(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearStyle;
	*n = XtNumber(uXrtGearStyle);
	return(*n);
}

int
UxValuesOfXrtGearType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearType;
	*n = XtNumber(uXrtGearType);
	return(*n);
}

/* New enums in Gear 2.0*/
int
UxValuesOfXrtGearArrowType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearArrowType;
	*n = XtNumber(uXrtGearArrowType);
	return(*n);
}

int
UxValuesOfXrtGearBorderLocation(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearBorderLocation;
	*n = XtNumber(uXrtGearBorderLocation);
	return(*n);
}

int
UxValuesOfXrtGearColumnResizePolicy(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearColumnResizePolicy;
	*n = XtNumber(uXrtGearColumnResizePolicy);
	return(*n);
}

int
UxValuesOfXrtGearDisplayPolicy(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearDisplayPolicy;
	*n = XtNumber(uXrtGearDisplayPolicy);
	return(*n);
}

int
UxValuesOfXrtGearImportPolicy(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearImportPolicy;
	*n = XtNumber(uXrtGearImportPolicy);
	return(*n);
}

int
UxValuesOfXrtGearIndentPolicy(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearIndentPolicy;
	*n = XtNumber(uXrtGearIndentPolicy);
	return(*n);
}

int
UxValuesOfXrtGearLineStyle(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearLineStyle;
	*n = XtNumber(uXrtGearLineStyle);
	return(*n);
}

int
UxValuesOfXrtGearMenuAlignment(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearMenuAlignment;
	*n = XtNumber(uXrtGearMenuAlignment);
	return(*n);
}

int
UxValuesOfXrtGearNodeMovePolicy(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearNodeMovePolicy;
	*n = XtNumber(uXrtGearNodeMovePolicy);
	return(*n);
}

int
UxValuesOfXrtGearProgressType(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearProgressType;
	*n = XtNumber(uXrtGearProgressType);
	return(*n);
}

int
UxValuesOfXrtGearSelectionPolicy(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearSelectionPolicy;
	*n = XtNumber(uXrtGearSelectionPolicy);
	return(*n);
}

int
UxValuesOfXrtGearSearchPolicy(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearSearchPolicy;
	*n = XtNumber(uXrtGearSearchPolicy);
	return(*n);
}

int
UxValuesOfXrtGearSizingPolicy(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearSizingPolicy;
	*n = XtNumber(uXrtGearSizingPolicy);
	return(*n);
}

int
UxValuesOfXrtGearTextClipPolicy(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearTextClipPolicy;
	*n = XtNumber(uXrtGearTextClipPolicy);
	return(*n);
}

int
UxValuesOfXrtGearWidgetResizePolicy(ulist, n)
char ***ulist;
int  *n;
{
	*ulist = uXrtGearWidgetResizePolicy;
	*n = XtNumber(uXrtGearWidgetResizePolicy);
	return(*n);
}

/* New X types in Gear 2.0*/
int
UxValuesOfXrtGearString(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0]= CGETS_KL(VALOF_GEARSTRING);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtGearIcon(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0]= CGETS_KL(VALOF_GEARICON);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtGearFolderStateMask(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0]= CGETS_KL(VALOF_GEARFOLDERSTATEMASK);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtGearSelectionRestrictions(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0]= CGETS_KL(VALOF_GEARSELECTIONRESTRICTIONS);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValuesOfXrtGearPickListIndex(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0]= CGETS_KL(VALOF_GEARPICKLISTINDEX);
	*s = values;
        *n = 1;
	return 0;
}

/*
 * Validate functions for both enumerated types and other Xtypes.
 * Use a general function for enumerated types.
 */
static int
ValidateXrtGearEnumType(rs, s, string_list, n)
swidget rs;
char	*s;
char	**string_list;
int		n;
{
	int i;

	if (s == NULL) {
		return ERROR;
	}
	for (i = 0; i < n; i++) {
		if (UxStrEqual(s, string_list[i])) {
			return NO_ERROR;
		}
	}
	return ERROR;
}

int
UxValidateXrtGearAlignerAlignment(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearAlignerAlignment,
								  XtNumber(uXrtGearAlignerAlignment)));
}

int
UxValidateXrtGearAlignment(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearAlignment,
								  XtNumber(uXrtGearAlignment)));
}

int
UxValidateXrtGearAnchorSide(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearAnchorSide,
								  XtNumber(uXrtGearAnchorSide)));
}

int
UxValidateXrtGearBorderType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearBorderType,
								  XtNumber(uXrtGearBorderType)));
}

int
UxValidateXrtGearChildType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearChildType,
								  XtNumber(uXrtGearChildType)));
}

int
UxValidateXrtGearIndicator(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearIndicator,
								  XtNumber(uXrtGearIndicator)));
}

int
UxValidateXrtGearPageAttachPolicy(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearPageAttachPolicy,
								  XtNumber(uXrtGearPageAttachPolicy)));
}

int
UxValuesOfXrtGearPSFontMapList(s, n)
char ***s;
int  *n;
{
	static char *values[1];

	values[0] = CGETS_KL(VALOF_GEARPSFONTMAPLIST);
	*s = values;
	*n = 1;
	return 0;
}

int
UxValidateXrtGearRotation(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearRotation,
								  XtNumber(uXrtGearRotation)));
}

int
UxValidateXrtGearTabSide(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearTabSide,
								  XtNumber(uXrtGearTabSide)));
}

int
UxValidateXrtGearStringLayout(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearStringLayout,
								  XtNumber(uXrtGearStringLayout)));
}

int
UxValidateXrtGearStyle(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearStyle,
								  XtNumber(uXrtGearStyle)));
}

int
UxValidateXrtGearType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearType,
								  XtNumber(uXrtGearType)));
}

/* New enums in Gear 2.0*/
int
UxValidateXrtGearArrowType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearArrowType,
								  XtNumber(uXrtGearArrowType)));
}

int
UxValidateXrtGearBorderLocation(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearBorderLocation,
								  XtNumber(uXrtGearBorderLocation)));
}

int
UxValidateXrtGearColumnResizePolicy(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearColumnResizePolicy,
								  XtNumber(uXrtGearColumnResizePolicy)));
}

int
UxValidateXrtGearDisplayPolicy(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearDisplayPolicy,
								  XtNumber(uXrtGearDisplayPolicy)));
}

int
UxValidateXrtGearImportPolicy(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearImportPolicy,
								  XtNumber(uXrtGearImportPolicy)));
}

int
UxValidateXrtGearIndentPolicy(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearIndentPolicy,
								  XtNumber(uXrtGearIndentPolicy)));
}

int
UxValidateXrtGearLineStyle(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearLineStyle,
								  XtNumber(uXrtGearLineStyle)));
}

int
UxValidateXrtGearMenuAlignment(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearMenuAlignment,
								  XtNumber(uXrtGearMenuAlignment)));
}

int
UxValidateXrtGearNodeMovePolicy(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearNodeMovePolicy,
								  XtNumber(uXrtGearNodeMovePolicy)));
}

int
UxValidateXrtGearProgressType(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearProgressType,
								  XtNumber(uXrtGearProgressType)));
}

int
UxValidateXrtGearSelectionPolicy(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearSelectionPolicy,
								  XtNumber(uXrtGearSelectionPolicy)));
}

int
UxValidateXrtGearSearchPolicy(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearSearchPolicy,
								  XtNumber(uXrtGearSearchPolicy)));
}

int
UxValidateXrtGearSizingPolicy(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearSizingPolicy,
								  XtNumber(uXrtGearSizingPolicy)));
}

int
UxValidateXrtGearWidgetResizePolicy(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearWidgetResizePolicy,
								  XtNumber(uXrtGearWidgetResizePolicy)));
}

int
UxValidateXrtGearTextClipPolicy(rs, s)
swidget rs;
char *s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearTextClipPolicy,
								  XtNumber(uXrtGearTextClipPolicy)));
}

int
UxValuesOfXrtGearEditType(ulist, n)
char ***ulist;
int	*n;
{
	*ulist = uXrtGearEditType;
	*n = XtNumber(uXrtGearEditType);
	return(*n);
}

int
UxValidateXrtGearEditType(rs, s)
swidget	rs;
char	*s;
{
	return(ValidateXrtGearEnumType(rs, s, uXrtGearEditType,
								  XtNumber(uXrtGearEditType)));
}

/* New X types in Gear 2.0*/

int
UxValidateXrtGearPickListIndex(rs, count)
swidget rs;
int   count;
{
	Resource_t* itemcountRD = UxFindResource(rs,
				"xrtGearPickListIndex_Combo");
	Resource_t* listItemRD = UxFindResource(rs,"xrtGearPickList_Combo");
	char*	 items;

	if ( UxFindValue(rs, listItemRD, &items) == FALSE)
	{
	    return NO_ERROR;	/*-------------------------------------
				 * The widget will accept anything
				 * for a count if there aren't
				 * any items. Go figure
				 *-----------------------------------*/
	}
	else
	{
	    int actualcount;

	    actualcount = UxGetStringItemCount(items);

	    if (count < actualcount)
		return NO_ERROR;
	    return ERROR;
	}
}


#endif   /* ! RUNTIME */

/*
 *	Conversion functions TO and FROM UIMX
 */

static int
String_Mask(swgt, string, mask, flag, xtype)
swidget				swgt;
char				**string;
Pixmap				*mask;
int					flag;
int					xtype;
{
	int				status;
	XrmValue        from, to;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 *  mask -> string.
		 */

		info = FindXrtStringInfo(String_Mask, xtype, swgt);
		if (info) {
			*string = XtNewString(info->string);
		}
		else {
			*string = XtNewString("");
		}
		status = NO_ERROR;
	}
	else {
		/*
		 *  string -> mask
		 */
		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(Pixmap);
		to.addr = (caddr_t) mask;

		if (XtConvertAndStore(UxTopLevel,
							  XtRString, &from,
							  XmRXrtGearMask, &to))
		{
			status = NO_ERROR;
			RegisterXrtStringInfo(String_Mask, xtype, swgt, *string);
		}
	}
	
	return(status);
}

static int
String_Pixmap(swgt, string, pixmap, flag, xtype)
swidget				swgt;
char				**string;
Pixmap				*pixmap;
int					flag;
int					xtype;
{
	int				status;
	XrmValue        from, to;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 *  pixmap -> string.
		 */

		info = FindXrtStringInfo(String_Pixmap, xtype, swgt);
		if (info) {
			*string = XtNewString(info->string);
		}
		else {
			*string = XtNewString("");
		}

		status = NO_ERROR;
	}
	else {
		/*
		 *  string -> pixmap
		 */
		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(Pixmap);
		to.addr = (caddr_t) pixmap;

		if (XtConvertAndStore(UxTopLevel,
							  XtRString, &from,
							  XmRXrtGearPixmap, &to))
		{
			status = NO_ERROR;
			RegisterXrtStringInfo(String_Pixmap, xtype, swgt, *string);
		}
	}
	
	return(status);
}

static int
String_ArmMask(swgt, string, armMask, flag, xtype)
swidget				swgt;
char				**string;
Pixmap				*armMask;
int					flag;
int					xtype;
{
	int				status;
	XrmValue        from, to;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 *  armMask -> string.
		 */

		info = FindXrtStringInfo(String_ArmMask, xtype, swgt);
		if (info) {
			*string = XtNewString(info->string);
		}
		else {
			*string = XtNewString("");
		}
		status = NO_ERROR;
	}
	else {
		/*
		 *  string -> armMask
		 */
		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(Pixmap);
		to.addr = (caddr_t) armMask;

		if (XtConvertAndStore(UxTopLevel,
							  XtRString, &from,
							  XmRXrtGearMask, &to))
		{
			status = NO_ERROR;
			RegisterXrtStringInfo(String_ArmMask, xtype, swgt, *string);
		}
	}
	
	return(status);
}

static int
String_ArmPixmap(swgt, string, armPixmap, flag, xtype)
swidget				swgt;
char				**string;
Pixmap				*armPixmap;
int					flag;
int					xtype;
{
	int				status;
	XrmValue        from, to;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 *  armPixmap -> string.
		 */

		info = FindXrtStringInfo(String_ArmPixmap, xtype, swgt);
		if (info) {
			*string = XtNewString(info->string);
		}
		else {
			*string = XtNewString("");
		}
		status = NO_ERROR;
	}
	else {
		/*
		 *  string -> armPixmap
		 */
		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(Pixmap);
		to.addr = (caddr_t) armPixmap;

		if (XtConvertAndStore(UxTopLevel,
							  XtRString, &from,
							  XmRXrtGearPixmap, &to))
		{
			status = NO_ERROR;
			RegisterXrtStringInfo(String_ArmPixmap, xtype, swgt, *string);
		}
	}
	
	return(status);
}

static int
Name_Widget(swgt, name, wgt, flag, xtype)
swidget				swgt;
char				**name;
Widget				*wgt;
int					flag;
int					xtype;
{
	swidget			named_swgt;
	int				status;
	XrmValue        from, to;
	XrtStringInfo	*info;

	status = NO_ERROR;

	if (flag == TO_UIMX) {
		/*
		 *  widget -> name.
		 */

		if(*wgt != NULL)
			*name = XtName(*wgt);
		else
			*name = "";


	}
	else {
		/*
		 *  name -> widget
		 */

		
		named_swgt = UxNameToSwidget(swgt, *name);
		if ( named_swgt == NULL )
			*wgt = NULL;
		else
			*wgt = UxGetWidget(named_swgt);

		if (*wgt == NULL)
			status = ERROR;
	}
	return(status);
}

static int
String_IndicatorPixmapList(swgt, string, indicatorPixmapList, flag, xtype)
swidget				swgt;
char				**string;
char				***indicatorPixmapList;
int					flag;
int					xtype;
{
	int				status;
	XrmValue        from, to;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 *  indicatorPixmapList -> string.
		 */


		info = FindXrtStringInfo(String_IndicatorPixmapList, xtype, swgt);
		if (info) {
			*string = XtNewString(info->string);
		}
		else {
			*string = XtNewString("");
		}
		status = NO_ERROR;
	}
	else {
		/*
		 *  string -> indicatorPixmapList
		 */

		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(XtPointer);
		to.addr = (caddr_t) indicatorPixmapList;

		if (XtConvertAndStore(UxTopLevel,
							  XtRString, &from,
							  XmRXrtGearIndicatorPixmapList, &to))
		{
			status = NO_ERROR;
			RegisterXrtStringInfo(String_IndicatorPixmapList, xtype, swgt, *string);
		}
	}
	
	return(status);
}

static int
String_IndicatorMaskList(swgt, string, indicatorMaskList, flag, xtype)
swidget				swgt;
char				**string;
char				***indicatorMaskList;
int					flag;
int					xtype;
{
	int				status;
	XrmValue        from, to;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 *  indicatorMaskList -> string.
		 */


		info = FindXrtStringInfo(String_IndicatorMaskList, xtype, swgt);
		if (info) {
			*string = XtNewString(info->string);
		}
		else {
			*string = XtNewString("");
		}
		status = NO_ERROR;
	}
	else {
		/*
		 *  string -> indicatorMaskList
		 */

		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(XtPointer);
		to.addr = (caddr_t) indicatorMaskList;

		if (XtConvertAndStore(UxTopLevel,
							  XtRString, &from,
							  XmRXrtGearIndicatorMaskList, &to))
		{
			status = NO_ERROR;
			RegisterXrtStringInfo(String_IndicatorMaskList, xtype, swgt, *string);
		}
	}
	
	return(status);
}

/* New X types in Gear 2.0*/
static int
String_SelectionRestrictions(swgt, string, restrictions, flag, xtype)
swidget		  				  swgt;
char						  **string;
XrtGearSelectionRestrictions  *restrictions;
int							  flag;
int						 	  xtype;
{
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 * XrtGearSelctionRestriction -> string
		 */
		
		from.size = sizeof(XrtGearSelectionRestrictions); /*??*/
		from.addr = (caddr_t) restrictions;

		to.addr = NULL;

		if (XtConvertAndStore(UxTopLevel,
							  XmRXrtGearSelectionRestrictions, &from,
							  XmRString, &to))
		{
 			*string = to.addr; 
			status = NO_ERROR;
		}
	}
	else {
		/*
		 * string -> XrtGearSelctionRestriction
		 */

		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(XrtGearSelectionRestrictions);
		to.addr = (caddr_t) restrictions;

		if (XtConvertAndStore(UxTopLevel,
							  XmRString, &from,
							  XmRXrtGearSelectionRestrictions, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}

static int
String_FolderStateMask(swgt, string, mask, flag, xtype)
swidget	swgt;
char  	**string;
int		*mask;
int		flag;
int		xtype;
{
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 * XrtGearFolderStateMask -> string
		 */
		
		from.size = sizeof(int);
		from.addr = (caddr_t) mask;

		to.addr = NULL;

		if (XtConvertAndStore(UxTopLevel,
							  XmRXrtGearFolderStateMask, &from,
							  XmRString, &to))
		{
 			*string = to.addr;
			  
			status = NO_ERROR;
		}
	}
	else {
		/*
		 * string -> XrtGearFolderStateMask
		 */

		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(int);
		to.addr = (caddr_t) mask;

		if (XtConvertAndStore(UxTopLevel,
							  XmRString, &from,
							  XmRXrtGearFolderStateMask, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}

static int
String_GearString(swgt, string, gearString, flag, xtype)
swidget			swgt;
char			**string;
XrtGearObject	*gearString;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;
	XrtStringInfo	*info;

	status = ERROR;

#if	0
	/* The following codes seem to work, but it is not
	   necessity to substitute them with existing codes... */
	if (flag == TO_UIMX) {
		if(!gearString || !*gearString)
		  *string = XtNewString("");
		else{
		  *string = XtNewString(XrtGearStringGetValue(*gearString));
		}
		status = NO_ERROR;
	}
	else {
		/*
		 *  Turn string -> GearString
		 */

		if(!string || !*string)
		  *gearString = XrtGearStringCreateCharString("");
		else
		  *gearString = XrtGearStringCreateCharString(*string);
		status = NO_ERROR;
	}
#else
	if (flag == TO_UIMX) {
		
		info = FindXrtStringInfo(String_GearString, xtype, swgt);
		if (info) {
			*string = XtNewString(info->string);
		}
		else {
			*string = XtNewString("");
		}
		status = NO_ERROR;
	}
	else {
		/*
		 *  Turn string -> GearString
		 */

		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(XrtGearObject);
		to.addr = (caddr_t) gearString;

		if (XtConvertAndStore(UxTopLevel,
							  XmRString, &from,
							  XmRXrtGearString, &to))
		{
			status = NO_ERROR;
			RegisterXrtStringInfo(String_GearString, xtype, swgt, *string);
		}
	}
#endif

done:
	return(status);
}

static int
String_StringTable(swgt, string, string_list, flag, xtype)
swidget			swgt;
char			**string;
char			***string_list;
int				flag;
int				xtype;
{
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {
		
		from.size = sizeof(char **);
		from.addr = (caddr_t) string_list;

		to.addr = NULL;


 		if (XtConvertAndStore(UxTopLevel, 
 							  XmRStringTable, &from, 
 							  XmRString, &to)) 
 		{ 
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
 		} 
	}
	else {
		/*
		 *  Turn a string list into a string table
		 */

		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(char **);
		to.addr = (caddr_t) string_list;

		if (XtConvertAndStore(UxTopLevel,
							  XmRString, &from,
							  XmRStringTable, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}

static int
String_Icon(swgt, string, icon, flag, xtype)
swidget				swgt;
char				**string;
XrtGearIcon			**icon;
int					flag;
int					xtype;
{
	int				status;
	XrmValue        from, to;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 *  icon -> string
		 */

		from.size = sizeof(XrtGearIcon *);
		from.addr = (caddr_t) icon;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XmRXrtGearIcon, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			status = NO_ERROR;
		}
	}
	else {	  
		/*
		 *  string -> icon
		 */

		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(XrtGearIcon);
		to.addr = (caddr_t) XtNew(XrtGearIcon);

		((XrtGearIcon *)to.addr)->icon      = XmUNSPECIFIED_PIXMAP;
		((XrtGearIcon *)to.addr)->icon_mask = XmUNSPECIFIED_PIXMAP;

		if (XtConvertAndStore(UxTopLevel, XmRString, &from, XmRXrtGearIcon, &to)) {
			*icon =  (XrtGearIcon *)to.addr;
			status = NO_ERROR;
		}
	}
	
	return(status);
}

static int
String_Cursor(swgt, string, cursor, flag, xtype)
swidget				swgt;
char				**string;
Cursor				*cursor;
int					flag;
int					xtype;
{
	int				status;
	XrmValue        from, to;
	XrtStringInfo	*info;

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 *  cursor -> string.
		 */

		info = FindXrtStringInfo(String_Cursor, xtype, swgt);
		if (info) {
			*string = XtNewString(info->string);
		}
		else {
			*string = XtNewString("");
		}
		status = NO_ERROR;
	}
	else {
		/*
		 *  string -> cursor
		 */
		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(Cursor);
		to.addr = (caddr_t) cursor;

		if (XtConvertAndStore(UxTopLevel,
							  XtRString, &from,
							  XmRCursor, &to))
		{
			status = NO_ERROR;
			RegisterXrtStringInfo(String_Cursor, xtype, swgt, *string);
		}
	}
	
	return(status);
}

static int
String_PSFontMapList(swgt, string, PSFontMap, flag, xtype)
swidget			swgt;
char			**string;
XrtGearPSFontMap	**PSFontMap;
int				flag;
int				xtype;
{
	int			i;
	int			status;
	XrmValue	from, to;

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 *  Turn a PSFontMapList into a string.
		 */
		

		from.size = sizeof(XrtGearPSFontMap *);
		from.addr = (caddr_t) PSFontMap;

		to.addr = NULL;

		if (XtConvertAndStore (UxTopLevel,
							   XmRXrtGearPSFontMapList, &from,
							   XtRString, &to))
		{
			*string = XrtEscapeQuotes(to.addr);
			if (to.addr) {
				XtFree(to.addr);
			}
			status = NO_ERROR;
		}
	}
	else {
		/*
		 *  Turn string into a PSFontMapList.
		 */
		
		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(XrtGearPSFontMap *);
		to.addr = (caddr_t) PSFontMap;

		if (XtConvertAndStore (UxTopLevel,
							   XtRString, &from,
							   XmRXrtGearPSFontMapList, &to))
		{
			status = NO_ERROR;
		}
	}

	return(status);
}

static int
String_Ulong(swgt, string, ul, flag, xtype)
swidget				swgt;
char			  	**string;
unsigned long		*ul;
int					flag;
int					xtype;
{
	int				status;
	XrmValue        from, to;

	status = ERROR;

	if (flag == TO_UIMX) {
		/*
		 * ULong -> string
		 */
		
		from.size = sizeof(unsigned long);
		from.addr = (caddr_t) ul;

		to.addr = NULL;

		if (XtConvertAndStore(UxTopLevel,
							  XmRXrtGearUlong, &from,
							  XmRString, &to))
		{
			*string = to.addr;
			status = NO_ERROR;
		}
	}
	else {
		/*
		 * string -> XrtGearSelctionRestriction
		 */

		
		from.size = strlen( *string ) + 1;
		from.addr = *string;

		to.size = sizeof(unsigned long);
		to.addr = (caddr_t) ul;

		if (XtConvertAndStore(UxTopLevel,
							  XmRString, &from,
							  XmRXrtGearUlong, &to))
		{
			status = NO_ERROR;
		}
	}

done:
	return(status);
}

void
XrtGear_enum()
{
	UxXT_XrtGearAlignerAlignment =
		UxAddEnumType(XmRXrtGearAlignerAlignment, 
  					  sizeof(unsigned char),
					  xXrtGearAlignerAlignment, 
					  uXrtGearAlignerAlignment, 
					  dXrtGearAlignerAlignment, 
					  XtNumber(uXrtGearAlignerAlignment), 
					  UxStringToCharEnum); 
	UxXT_XrtGearAlignment =
		UxAddEnumType(XmRXrtGearAlignment, 
  					  sizeof(unsigned char),
					  xXrtGearAlignment, 
					  uXrtGearAlignment, 
					  dXrtGearAlignment, 
					  XtNumber(uXrtGearAlignment), 
					  UxStringToCharEnum); 
	UxXT_XrtGearAnchorSide =
		UxAddEnumType(XmRXrtGearAnchorSide, 
  					  sizeof(unsigned char),
					  xXrtGearAnchorSide, 
					  uXrtGearAnchorSide, 
					  dXrtGearAnchorSide, 
					  XtNumber(uXrtGearAnchorSide), 
					  UxStringToCharEnum); 
	UxXT_XrtGearBorderType =
		UxAddEnumType(XmRXrtGearBorderType, 
  					  sizeof(unsigned char),
					  xXrtGearBorderType, 
					  uXrtGearBorderType, 
					  dXrtGearBorderType, 
					  XtNumber(uXrtGearBorderType), 
					  UxStringToCharEnum); 
	UxXT_XrtGearChildType =
		UxAddEnumType(XmRXrtGearChildType, 
  					  sizeof(unsigned char),
					  xXrtGearChildType, 
					  uXrtGearChildType, 
					  dXrtGearChildType, 
					  XtNumber(uXrtGearChildType), 
					  UxStringToCharEnum); 
	UxXT_XrtGearIndicator =
		UxAddEnumType(XmRXrtGearIndicator, 
  					  sizeof(unsigned char),
					  xXrtGearIndicator, 
					  uXrtGearIndicator, 
					  dXrtGearIndicator, 
					  XtNumber(uXrtGearIndicator), 
					  UxStringToCharEnum); 
	UxXT_XrtGearPageAttachPolicy =
		UxAddEnumType(XmRXrtGearPageAttachPolicy, 
  					  sizeof(unsigned char),
					  xXrtGearPageAttachPolicy, 
					  uXrtGearPageAttachPolicy, 
					  dXrtGearPageAttachPolicy, 
					  XtNumber(uXrtGearPageAttachPolicy), 
					  UxStringToCharEnum); 
	UxXT_XrtGearRotation =
		UxAddEnumType(XmRXrtGearRotation, 
  					  sizeof(unsigned char),
					  xXrtGearRotation, 
					  uXrtGearRotation, 
					  dXrtGearRotation, 
					  XtNumber(uXrtGearRotation), 
					  UxStringToCharEnum); 
	UxXT_XrtGearTabSide =
		UxAddEnumType(XmRXrtGearTabSide, 
  					  sizeof(unsigned char),
					  xXrtGearTabSide, 
					  uXrtGearTabSide, 
					  dXrtGearTabSide, 
					  XtNumber(uXrtGearTabSide), 
					  UxStringToCharEnum); 
	UxXT_XrtGearStringLayout =
		UxAddEnumType(XmRXrtGearStringLayout, 
  					  sizeof(unsigned char),
					  xXrtGearStringLayout, 
					  uXrtGearStringLayout, 
					  dXrtGearStringLayout, 
					  XtNumber(uXrtGearStringLayout), 
					  UxStringToCharEnum); 
	UxXT_XrtGearStyle =
		UxAddEnumType(XmRXrtGearStyle, 
  					  sizeof(unsigned char),
					  xXrtGearStyle, 
					  uXrtGearStyle, 
					  dXrtGearStyle, 
					  XtNumber(uXrtGearStyle), 
					  UxStringToCharEnum); 
	UxXT_XrtGearType =
		UxAddEnumType(XmRXrtGearType, 
  					  sizeof(unsigned char),
					  xXrtGearType, 
					  uXrtGearType, 
					  dXrtGearType, 
					  XtNumber(uXrtGearType), 
					  UxStringToCharEnum); 
/* New enums in Gear 2.0*/
	UxXT_XrtGearArrowType =
		UxAddEnumType(XmRXrtGearArrowType,
  					  sizeof(unsigned char),
					  xXrtGearArrowType, 
					  uXrtGearArrowType, 
					  dXrtGearArrowType, 
					  XtNumber(uXrtGearArrowType), 
					  UxStringToCharEnum);
	UxXT_XrtGearBorderLocation =
		UxAddEnumType(XmRXrtGearBorderLocation,
  					  sizeof(unsigned char),
					  xXrtGearBorderLocation, 
					  uXrtGearBorderLocation, 
					  dXrtGearBorderLocation, 
					  XtNumber(uXrtGearBorderLocation), 
					  UxStringToCharEnum);
	UxXT_XrtGearColumnResizePolicy =
		UxAddEnumType(XmRXrtGearColumnResizePolicy, 
  					  sizeof(unsigned char),
					  xXrtGearColumnResizePolicy, 
					  uXrtGearColumnResizePolicy, 
					  dXrtGearColumnResizePolicy, 
					  XtNumber(uXrtGearColumnResizePolicy), 
					  UxStringToCharEnum); 
	UxXT_XrtGearDisplayPolicy =
		UxAddEnumType(XmRXrtGearDisplayPolicy, 
  					  sizeof(unsigned char),
					  xXrtGearDisplayPolicy, 
					  uXrtGearDisplayPolicy, 
					  dXrtGearDisplayPolicy, 
					  XtNumber(uXrtGearDisplayPolicy), 
					  UxStringToCharEnum); 
	UxXT_XrtGearImportPolicy =
		UxAddEnumType(XmRXrtGearImportPolicy, 
  					  sizeof(unsigned char),
					  xXrtGearImportPolicy, 
					  uXrtGearImportPolicy, 
					  dXrtGearImportPolicy, 
					  XtNumber(uXrtGearImportPolicy), 
					  UxStringToCharEnum); 
	UxXT_XrtGearIndentPolicy =
		UxAddEnumType(XmRXrtGearIndentPolicy, 
  					  sizeof(unsigned char),
					  xXrtGearIndentPolicy, 
					  uXrtGearIndentPolicy, 
					  dXrtGearIndentPolicy, 
					  XtNumber(uXrtGearIndentPolicy), 
					  UxStringToCharEnum); 
	UxXT_XrtGearLineStyle =
		UxAddEnumType(XmRXrtGearLineStyle, 
  					  sizeof(unsigned char),
					  xXrtGearLineStyle, 
					  uXrtGearLineStyle, 
					  dXrtGearLineStyle, 
					  XtNumber(uXrtGearLineStyle), 
					  UxStringToCharEnum); 
	UxXT_XrtGearMenuAlignment =
		UxAddEnumType(XmRXrtGearMenuAlignment,
  					  sizeof(unsigned char),
					  xXrtGearMenuAlignment, 
					  uXrtGearMenuAlignment, 
					  dXrtGearMenuAlignment, 
					  XtNumber(uXrtGearMenuAlignment), 
					  UxStringToCharEnum);
	UxXT_XrtGearNodeMovePolicy =
		UxAddEnumType(XmRXrtGearMovePolicy, 
  					  sizeof(unsigned char),
					  xXrtGearNodeMovePolicy, 
					  uXrtGearNodeMovePolicy, 
					  dXrtGearNodeMovePolicy, 
					  XtNumber(uXrtGearNodeMovePolicy), 
					  UxStringToCharEnum); 
	UxXT_XrtGearProgressType =
		UxAddEnumType(XmRXrtGearProgressType,
  					  sizeof(unsigned char),
					  xXrtGearProgressType, 
					  uXrtGearProgressType, 
					  dXrtGearProgressType, 
					  XtNumber(uXrtGearProgressType), 
					  UxStringToCharEnum);
	UxXT_XrtGearSelectionPolicy =
		UxAddEnumType(XmRXrtGearSelectionPolicy, 
  					  sizeof(unsigned char),
					  xXrtGearSelectionPolicy, 
					  uXrtGearSelectionPolicy, 
					  dXrtGearSelectionPolicy, 
					  XtNumber(uXrtGearSelectionPolicy), 
					  UxStringToCharEnum); 
	UxXT_XrtGearSearchPolicy =
		UxAddEnumType(XmRXrtGearSearchPolicy, 
  					  sizeof(unsigned char),
					  xXrtGearSearchPolicy, 
					  uXrtGearSearchPolicy, 
					  dXrtGearSearchPolicy, 
					  XtNumber(uXrtGearSearchPolicy), 
					  UxStringToCharEnum); 
	UxXT_XrtGearSizingPolicy =
		UxAddEnumType(XmRXrtGearSizingPolicy, 
  					  sizeof(unsigned char),
					  xXrtGearSizingPolicy, 
					  uXrtGearSizingPolicy, 
					  dXrtGearSizingPolicy, 
					  XtNumber(uXrtGearSizingPolicy), 
					  UxStringToCharEnum); 
	UxXT_XrtGearTextClipPolicy =
		UxAddEnumType(XmRXrtGearTextHorizClipPolicy, 
  					  sizeof(unsigned char),
					  xXrtGearTextClipPolicy, 
					  uXrtGearTextClipPolicy, 
					  dXrtGearTextClipPolicy, 
					  XtNumber(uXrtGearTextClipPolicy), 
					  UxStringToCharEnum); 
	UxXT_XrtGearWidgetResizePolicy =
		UxAddEnumType(XmRXrtGearWidgetResizePolicy, 
  					  sizeof(unsigned char),
					  xXrtGearWidgetResizePolicy, 
					  uXrtGearWidgetResizePolicy, 
					  dXrtGearWidgetResizePolicy, 
					  XtNumber(uXrtGearWidgetResizePolicy), 
					  UxStringToCharEnum); 

	UxXT_XrtGearEditType = UxAddEnumType(
					  XmRXrtGearEditType, 
					  sizeof(unsigned char),
					  xXrtGearEditType,
					  uXrtGearEditType,
					  dXrtGearEditType,
					  XtNumber(xXrtGearEditType),
					  UxStringToCharEnum);
}

/* Add new XRT/gear xtypes */
void
XrtGear_xtypes()
{
	/*
	 * Initialize widget class in order to initialize converters.  Also
	 * install reverse converters.
	 */
	XtInitializeWidgetClass(xmXrtAlignerWidgetClass);
	XtInitializeWidgetClass(xmXrtGLabelWidgetClass);
	XtInitializeWidgetClass(xmXrtPushButtonWidgetClass);
	XtInitializeWidgetClass(xmXrtTabButtonWidgetClass);
	XtInitializeWidgetClass(xmXrtTabManagerWidgetClass);
	XtInitializeWidgetClass(xmXrtToggleButtonWidgetClass);
	XtInitializeWidgetClass(xmXrtToolbarWidgetClass);

	XtInitializeWidgetClass(xmXrtOutlinerWidgetClass);
	XtInitializeWidgetClass(xmXrtNodeStyleObjectClass);
 	XtInitializeWidgetClass(xmXrtColumnObjectClass); 
 	XtInitializeWidgetClass(xmXrtGearComboWidgetClass); 
 	XtInitializeWidgetClass(xmXrtProgressWidgetClass); 
 	XtInitializeWidgetClass(xmXrtPanedWindowWidgetClass); 
	
	UxXT_XrtGearArmMask				= UxAddXtype(XmRXrtGearMask, sizeof(Pixmap));
	UxXT_XrtGearArmPixmap			= UxAddXtype(XmRXrtGearPixmap, sizeof(Pixmap));
	UxXT_XrtGearColumn				= UxAddXtype(XmRXrtGearColumn, sizeof(Widget));
	UxXT_XrtGearCursor				= UxAddXtype(XmRCursor, sizeof(Cursor));
	UxXT_XrtGearFolderStateMask		= UxAddXtype(XmRXrtGearFolderStateMask, sizeof(int));
	UxXT_XrtGearIcon				= UxAddXtype(XmRXrtGearIcon,  sizeof(XrtGearIcon));
	UxXT_XrtGearIndicatorPixmapList	= UxAddXtype(XmRXrtGearIndicatorPixmapList, sizeof(char**));
	UxXT_XrtGearIndicatorMaskList	= UxAddXtype(XmRXrtGearIndicatorMaskList, sizeof(char**));
	UxXT_XrtGearMask				= UxAddXtype(XmRXrtGearMask, sizeof(Pixmap));
	UxXT_XrtGearNodeStyle			= UxAddXtype(XmRXrtGearNodeStyle,sizeof(Widget));
	UxXT_XrtGearPixmap				= UxAddXtype(XmRXrtGearPixmap, sizeof(Pixmap));
	UxXT_XrtGearPSFontMapList		= UxAddXtype(XmRXrtGearPSFontMapList,
												 sizeof(XrtGearPSFontMap *));
	UxXT_XrtGearSelectionRestrictions = UxAddXtype(XmRXrtGearSelectionRestrictions,
												 sizeof(XrtGearSelectionRestrictions));
	UxXT_XrtGearString				= UxAddXtype(XmRXrtGearString, sizeof(XrtGearObject));
	UxXT_XrtGearStringTable			= UxAddXtype(XmRStringTable, sizeof(char **));
	UxXT_XrtGearUlong				= UxAddXtype(XmRXrtGearUlong, sizeof(unsigned long));
	UxXT_XrtGearWidget				= UxAddXtype(XmRXrtGearWidget, sizeof(Widget));

	/* Add new converters for Xtypes */
	UxAddConv(UxUT_string,UxXT_XrtGearArmMask,				String_ArmMask);
	UxAddConv(UxUT_string,UxXT_XrtGearArmPixmap,			String_ArmPixmap);
	UxAddConv(UxUT_string,UxXT_XrtGearColumn,				Name_Widget); /* use widget converter,??? */
	UxAddConv(UxUT_string,UxXT_XrtGearCursor,				String_Cursor);
	UxAddConv(UxUT_string,UxXT_XrtGearFolderStateMask,		String_FolderStateMask);
	UxAddConv(UxUT_string,UxXT_XrtGearIcon,					String_Icon);
	UxAddConv(UxUT_string,UxXT_XrtGearIndicatorMaskList,	String_IndicatorMaskList);
	UxAddConv(UxUT_string,UxXT_XrtGearIndicatorPixmapList,	String_IndicatorPixmapList);
	UxAddConv(UxUT_string,UxXT_XrtGearMask,					String_Mask);
	UxAddConv(UxUT_string,UxXT_XrtGearNodeStyle,			Name_Widget); /* let's see if it will work ?? */
	UxAddConv(UxUT_string,UxXT_XrtGearPixmap,				String_Pixmap);
	UxAddConv(UxUT_string,UxXT_XrtGearPSFontMapList,		String_PSFontMapList);
	UxAddConv(UxUT_string,UxXT_XrtGearSelectionRestrictions, String_SelectionRestrictions);
	UxAddConv(UxUT_string,UxXT_XrtGearString,				String_GearString);
	UxAddConv(UxUT_string,UxXT_XrtGearStringTable,			String_StringTable);
	UxAddConv(UxUT_string,UxXT_XrtGearUlong,				String_Ulong);
	UxAddConv(UxUT_string,UxXT_XrtGearWidget,				Name_Widget);
}














